#!/bin/sh
# wdimex.sh -- make the message file and test wdm files for glsnet
#
# History:  96/02/16 kmflynn
#
# Usage: wdimex.sh [ file_name ] [| tee output_file ]
#
# Where: file_name = the name of the message file or test wd file to build
#                    glsmes - to generate the glsnet message file (default)
#                    tx - generate texas peak flows wdm file
#                    va - generate virginia low flows wdm file
#
# Examples:  wdimex.sh
#            wdimex.sh va

# pathname and libraries
  WrdA=/usr/opt/wrdapp
  Libr=$WrdA/lib3.1
  Anne=$WrdA/annie2.3

#*******************************************************************
#***** You should not need to modify anything below this line. *****
#*******************************************************************

Name=${1-'glsmes'}
     if [ ! $Name ] ; then Name=glsmes ; fi

# gls message file
  Seqs='glsnet algmes erranl netwrk progls prosls prtpro prtste'
  SeqL='ann/qtdate    waide/agplot  waide/tbltmp'

  echo
  echo ' Building wdm files'
  echo '           library:' $Libr
  echo '          wdm file:' $Name.wdm

#   remove any old files
    if [ -f $Name.wdm ] ; then rm $Name.wdm ; fi
    if [ -f error.fil ] ; then rm error.fil ; fi
    if [ -f ERROR.FIL ] ; then rm ERROR.FIL ; fi
    if [ -f $Name.in ]  ; then rm $Name.in  ; fi
    if [ -f $Name.err ] ; then rm $Name.err ; fi

  if [ $Name = 'glsmes' ] ; then
#   build the message file
    echo '       sequentials:' $Seqs
    echo '                   ' $SeqL
    echo

#   build input file
    echo $Name.wdm > $Name.in
    for Seq in $Seqs ; do
       echo I >> $Name.in
       echo $Seq.seq >> $Name.in
    done
    for Seq in $SeqL ; do
       echo I >> $Name.in
       echo $Libr/msg/$Seq.seq >> $Name.in
    done
    echo R >> $Name.in

#   get message file shell
    cp $Libr/lib_data/message.wdm $Name.wdm

#   build message file
    $Libr/bin/wdimex < $Name.in

#   remove old message file and move new one into bin_data
    if [ -f ../bin_data/$Name.wdm ] ; then rm ../bin_data/$Name.wdm ; fi
    mv $Name.wdm ../bin_data
#   save error.fil file
    mv error.fil $Name.err

  else

#   build a test wdm file
    echo '        sequential:' ../data/$Name.exp
    echo

#   build import log file
    echo fb$Name.wdm > $Name.in
    echo '#402' >> $Name.in
    echo rai../data/$Name.exp >> $Name.in
    echo '#402' >> $Name.in
    echo '#402' >> $Name.in
    echo '#402' >> $Name.in
    echo rfs >> $Name.in
    echo '#402' >> $Name.in
    echo rrr >> $Name.in

#   build the test wdm file
    $Anne/bin/annie <<-EOT
	@$Name.in
	EOT

#   remove old data file and move new one into data
    rm XPAD.DAT ANNIE.LOG
    if [ -f ../data/$Name.wdm ] ; then rm ../data/$Name.wdm ; fi
    mv $Name.wdm ../data/$Name.wdm
    mv ERROR.FIL $Name.err

  fi
    
# end of shell
