C
C
C
      SUBROUTINE   DECOMP
     I                    (N,NDIM,XLAM,FO,NSIZ,
     O                     B)
C
C     + + + PURPOSE + + +
C     This routine for matrix inversion uses Cholesky's method.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   N, NDIM,  FO, NSIZ
      DOUBLE PRECISION   XLAM(NDIM,NDIM), B(NSIZ,NSIZ)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     N      - actual size of matrices for computations
C     NDIM   - dimension of XLAM array
C     XLAM   - square input matrix dimensioned NDIM
C     FO     - Fortran unit number of output file for errors
C     NSIZ   - size of B output array (max number of stations)
C     B      - square output matrix dimensioned NSIZ
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   IS, ISM, JS, JSM, KS
      DOUBLE PRECISION   BH, BN, DZERO
C
C     + + + INTRINSICS + + +
      INTRINSIC   DSQRT, DABS, DMAX1
C
C     + + + OUTPUT FORMATS + + +
 2001 FORMAT('ZERO VALUE FOR B(1,1) IN DECOMP. MUST STOP.')
 2002 FORMAT('ZERO VALUE FOR B(JS,JS) IN DECOMP. MUST STOP.')
 2096 FORMAT( ' IN DECOMP/ NDIM,XLAM 1-1,2-1,2-2,1-2 = ',I5,4F10.4,
     1/,' COVARIANCE MATRIX NOT POSITIVE DEFINITE')
C2097 FORMAT(1X,' B22=',F10.3)
 2098 FORMAT(1X,' COVARIANCE MATRIX NOT POSITIVE DEFINITE BN=',F10.3,
     1 ' I=',I4,'    J=',I4)
C2099 FORMAT(I5,/(10F10.5))
C
C     + + + END SPECIFICATIONS + + +
C
      DZERO = 1.0D-30
      IF(XLAM(1,1).LE.0..OR.XLAM(2,2).LE.0.)
     1  WRITE (FO,2096) NDIM,XLAM(1,1),XLAM(2,1),XLAM(2,2),XLAM(1,2)
      B(1,1)=DSQRT(XLAM(1,1))
      IF (DABS(B(1,1)) .LT. DZERO) THEN
        WRITE (FO,2001)
        STOP 'DECOMP1'
      END IF
      B(1,2)=0.
      B(2,1)=XLAM(2,1)/B(1,1)
C     WRITE (FO,2097)B(2,2)
      B(2,2)=DSQRT(XLAM(2,2)-B(2,1)**2)
C     WRITE(FO,2099) NDIM, B(1,1),B(2,1),B(2,2)
      IF(N.GT.2) THEN
        DO 40 IS=3,N
          B(IS,1)=XLAM(IS,1)/B(1,1)
          BN=XLAM(IS,IS)-B(IS,1)**2
          ISM=IS-1
          DO 30 JS=2,ISM
            JSM=JS-1
            BH=XLAM(IS,JS)
            DO 20 KS=1,JSM
              BH=BH-B(IS,KS)*B(JS,KS)
 20         CONTINUE
            IF (DABS(B(JS,JS)) .LT. DZERO) THEN
              WRITE (FO,2002)
              STOP 'DECOMP2'
            END IF
            B(IS,JS)=BH/B(JS,JS)
            BN=BN-B(IS,JS)**2
 30       CONTINUE
          IF( BN.LE.0.) WRITE (FO,2098)BN, IS, JS
          B(IS,IS)=DSQRT(DMAX1(BN, 0.D+0))
C         WRITE(FO,2099) N,BN,(B(IS,J), J=1,IS)
 40     CONTINUE
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   MULTPY
     I                    (X,Y,K1,K2,K3,N1,N2,N3,
     O                     PROD)
C
C     + + + PURPOSE + + +
C     This routine multiplies matrix X by Y and produces PROD.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   K1,K2,K3,N1,N2,N3
      DOUBLE PRECISION   PROD(N1,K3), X(N2,K2), Y(N3,K3)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     X      - K1 by K2 input matrix
C     Y      - K2 by K3 input matrix
C     K1     - rows in X and PROD matrices
C     K2     - columns in X and rows in Y matrices
C     K3     - columns in Y and PROD matrices
C     N1     - row dimension on PROD
C     N2     - dimensions on X
C     N3     - dimensions on Y
C     PROD   - product of X times Y
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I,J,K
      DOUBLE PRECISION   SUM
C
C     + + + END SPECIFICATIONS + + +
C
      DO 30 I=1,K1
        DO 20 K=1,K3
          SUM=0.
          DO 10 J=1,K2
            SUM=SUM+X(I,J)*Y(J,K)
 10       CONTINUE
          PROD(I,K)=SUM
 20     CONTINUE
 30   CONTINUE
C
      RETURN
      END
C
C
C
      SUBROUTINE   INVERT
     I                    (N,NDIM,COV,KSTAR,FO,
     O                     DET,COVINV)
C
C     + + + PURPOSE + + +
C     This routine computes determinent of COV then replaces COV with
C     its inverse, CONINV.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   N, NDIM, FO, KSTAR
      DOUBLE PRECISION    DET, COVINV(NDIM,NDIM), COV(NDIM,NDIM)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     N      - size of input matrix
C     NDIM   - dimension of input matrix
C     COV    - N by N input matrix
C     KSTAR  - ????
C     FO     - Fortran unit number for output file for error messages
C     DET    - determinent  ???
C     COVINV - inverse of COV matrix
C
C     + + + COMMON BLOCKS + + +
      INCLUDE 'pglsiz.inc'
C
C     + + + LOCAL COMMON + + +
      COMMON /LOCC/ A, B
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I, IM, J, K, STOPFG
      DOUBLE PRECISION   A(MAXPTS,MAXPTS), B(MAXPTS,MAXPTS),
     1                   SUM, TEMP
C
C     + + + INTRINSICS + + +
      INTRINSIC   DABS
C
C     + + + EXTERNALS + + +
      EXTERNAL   DECOMP
C
C     + + + OUTPUT FORMATS + + +
C2009 FORMAT(10F10.4)
 2000 FORMAT(1X,' PROCESSING STOPPED -- SINGULAR MATRIX')
C
C     + + + END SPECIFICATIONS + + +
C
C     B IS THE LOWER TRIANGULAR DECOMPOSITION OF COV
C
      STOPFG = 0
C
      IF (N.EQ.2) THEN
        DET = COV(1,1)*COV(2,2) - COV(1,2)**2
        TEMP = COV(1,1)/DET
        COVINV(1,1) = COV(2,2)/DET
        COVINV(2,2) = TEMP
        COVINV(1,2) = - COV(1,2)/DET
        COVINV(2,1) = COVINV(1,2)
      ELSE
C
        CALL DECOMP(N,NDIM,COV,FO,MAXPTS,B)
C       WRITE(FO,2009)((B(I,J),J=1,N), I=1,N)
C       DETL=B(1,1)
C       DO 10 I=2,N
C         DETL=DETL*B(I,I)
C10     CONTINUE
C       DET=DETL**2
C
        DO 11 I = 1,N
C         CHECK THAT DIAGONAL HAS NO 'ZEROS'.
          IF (DABS(B(I,I)) .LT. 1.0D-20) STOPFG = 1
 11     CONTINUE
C
        IF (STOPFG .EQ. 0) THEN
C         OK, CONTINUE
          A(1,1)=1./B(1,1)
          A(2,2)=1./B(2,2)
          A(2,1)=-B(2,1)*A(1,1)*A(2,2)
C
          DO 20 I=3,N
            A(I,I)=1./B(I,I)
            IM=I-1
            DO 15 K=1,IM
              SUM=0.
              DO 12 J=K,IM
                SUM=SUM+B(I,J)*A(J,K)
 12           CONTINUE
              A(I,K)=-SUM*A(I,I)
 15         CONTINUE
 20       CONTINUE
C
C         IF (KSTAR .EQ. 1) THEN
C           DO 35 I = 1,N
C             DO 30 J = 1,I
C               PINV(I,J) = A(I,J)
C30           CONTINUE
C35         CONTINUE
C         END IF
C
          DO 70 I=1,N
            DO 60 J=1,I
              SUM=0.
              DO 55 K=I,N
                SUM=SUM+A(K,I)*A(K,J)
 55           CONTINUE
              COVINV(I,J)=SUM
              COVINV(J,I)=SUM
 60         CONTINUE
 70       CONTINUE
        END IF
      END IF
C
      IF (STOPFG .EQ. 1) THEN
C       BAD NEWS, MATRIX WON'T INVERT, MUST KILL
        WRITE(FO,2000)
        STOP 'INVERT'
      END IF
C
      RETURN
      END
