C
C
C
      SUBROUTINE   PRTSTE
     I                   (MESSFL,WDMSFL,ERRFL,NSTAT,DSNP,NDAY)
C
C     + + + PURPOSE + + +
C     This routine adds data for a partial record site using the
C     Stedinger technique
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     MESSFL,WDMSFL,ERRFL,NSTAT,DSNP(NSTAT),NDAY(NSTAT)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MESSFL - Fortran unit number of message file
C     WDMSFL - Fortran unit number of WDM file
C     ERRFL  - Fortran unit number for diagnostic output
C     NSTAT  - Number of annual series for partial record
C              station in the buffer
C     DSNP   - Datasets of station for up to 12 annual low flow
C              series to add partial record estimates.
C     NDAY   - low flow durations for each data set
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      SCLU,SGRP,RESP,OFILE,ANAFG,RETCOD,IDXSTA(6),
     &             FLAG,NIDX,DSNI(6,12),NOBS,IYR(100)
      REAL   PRED(100),COOKD(100),RESID(100),YPR(100),XIS(100,6)
      DOUBLE PRECISION   XTXINV(10,10),B(10,1),SE
C
C     + + + EXTERNALS + + +
      EXTERNAL     QRESP,PRNTXT,QFOPEN,PSTIDX,PSTPLT,PSTANL,PSTCOM
C
C     + + + END SPECIFICATIONS + + +
C
C
C     cluster containing info for this code group
      SCLU = 153
C     assume no output file in use
      OFILE= 0
C     index sites not spec
C     analyze not done
      ANAFG= 0
C
      RESP = 1
 10   CONTINUE
C       main compute loop, output, input data, analysis,
C       plots, compute, done
        SGRP= 1
        CALL QRESP(MESSFL,SCLU,SGRP,
     M             RESP)
        IF (RESP .EQ. 1) THEN
C         specify output file
          SGRP= 2
          CALL QFOPEN(MESSFL,SCLU,SGRP,
     O                OFILE,RETCOD)
          IF (RETCOD .NE. 0) THEN
C           problem opening file
            WRITE(ERRFL,*) 'problem with output file',OFILE,RETCOD
            OFILE= 0
          END IF
        ELSE IF (RESP .EQ. 2) THEN
C         specify index sites and read data file
          CALL PSTIDX (MESSFL, SCLU, WDMSFL, NSTAT, DSNP, NDAY, 
     O                 NOBS, YPR, XIS, IYR, NIDX, DSNI, IDXSTA)
        ELSE IF (RESP .EQ. 3) THEN
C         analzye data at index sites
C         do analysis
          IF (OFILE .LE. 0) THEN
C           can't do analysis without an output file
            SGRP = 8
            CALL PRNTXT (MESSFL, SCLU, SGRP)
          ELSE IF (NOBS .LE. 0) THEN
C           can't do analysis without successfully reading input
            SGRP = 7
            CALL PRNTXT (MESSFL, SCLU, SGRP)
          ELSE
C           do analysis
            CALL PSTANL (MESSFL,SCLU,OFILE,NOBS,NIDX,IDXSTA,
     M                   YPR,XIS,
     #                   B,SE,XTXINV,RESID,COOKD,PRED,FLAG)
            IF (FLAG .EQ. 0) ANAFG= 1
          END IF
        ELSE IF (RESP .EQ. 4) THEN
C         plot results of analysis
          IF (ANAFG .EQ. 0) THEN
C           must do analysis first
            SGRP= 9
            CALL PRNTXT(MESSFL,SCLU,SGRP)
          ELSE
C           do plots
            CALL PSTPLT (MESSFL,SCLU,NOBS,YPR,PRED,RESID,COOKD)
          END IF
        ELSE IF (RESP .EQ. 5) THEN
C         compute and store results
          IF (ANAFG .EQ. 0) THEN
C           must do analysis first
            SGRP= 12
            CALL PRNTXT(MESSFL,SCLU,SGRP)
          ELSE
C           do it
            CALL PSTCOM (MESSFL,SCLU,WDMSFL,OFILE,NSTAT,NIDX,DSNI,DSNP,
     I                   B,SE,XTXINV)
            CALL PSTYRS (MESSFL, SCLU, WDMSFL, NSTAT, DSNP, NOBS, IYR)
          END IF
        END IF
      IF (RESP .NE. 6) GO TO 10
C
      RETURN
      END
C
C
C
      SUBROUTINE   PSTIDX
     I                  (MESSFL, SCLU, WDMSFL, NSTAT, DSNP, NDAY,
     O                   NOBS, YPR, XIS, IYR, NIDX, DSNI, IDXSTA)
C
C     + + + PURPOSE + + +
C     This routine opens the sequential input file of partial record and
C     index sites, reads that file and checks matches on station numbers.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   MESSFL, SCLU, WDMSFL, NSTAT, DSNP(NSTAT), NDAY(NSTAT), 
     &          NOBS, IYR(100), NIDX, DSNI(6,12), IDXSTA(6)
      REAL      YPR(100), XIS(100,6)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MESSFL - Fortran unit number of message file
C     SCLU   - cluster number on message file
C     WDMSFL - Fortran unit number of WDM file
C     NSTAT  - number of annual series for the station
C     DSNP   - Datasets of station for up to 12 annual low flow
C     NDAY   - 
C     NOBS   - number of concurrent discharges at partial and index
C              stations
C     YPR    - discharge at partial record station
C     XIS    - discharge at index sites
C     IYR    - calendar year of discharge measurement
C     NIDX   - number of index sites
C     DSNI   - data set numbers at index sites
C     IDXSTA - station numbers of index sites
C
C     + + + LOCAL VARIABLES + + +
      INTEGER       SGRP, INUNIT, RETCOD, TNIDX, K, FLAG, L1, L2, L3, 
     &              IVAL(10), FLG, ORDER(6), I, J, IRET, DATE(3),
     &              HDERR, IBAD, RESP
      REAL          PX(10)
      CHARACTER*1   CSTA,TSTYPE(4,12)
      CHARACTER*24  AVST
      CHARACTER*78  C78
      CHARACTER*112 STR
C
C     + + + EXTERNALS + + +
      EXTERNAL   QFOPEN, QRESPI, PRNTXT, ZIPI, GETTXT, CVARAR
      EXTERNAL   QRESPM, GTDSNS, ZSTADD, Q1INIT, Q1EDIT
      EXTERNAL   QGETI, QGETIB, QRESP, PRNTXI, QSETIB, ZMNSST
C
C     + + + FORMATS + + +
 1000 FORMAT (1X,I4,2I2,4X,10F9.0)
 1001 FORMAT (A1,21X,10I9)
 1002 FORMAT (122A1)
C
C     + + + DATA INITIALIZATION + + +
      DATA AVST/'Available index stations'/
C
C     + + + END SPECIFICATIONS + + +
      L1 = 1
      L2 = 2
      L3 = 3
      NOBS = 0
C     get input file
      SGRP = 99
      CALL QFOPEN (MESSFL,SCLU,SGRP,INUNIT,RETCOD)
      IF (RETCOD .EQ. 0) THEN
C       got an inout file opened
C       Number of index sites on file
        TNIDX = 10
C       read header record off file
        K = 22 + TNIDX*9
        READ(INUNIT,1002,IOSTAT=HDERR) (STR(I:I),I=1,K)
        READ (STR,1001,IOSTAT=HDERR)CSTA,(IVAL(J),J=1,TNIDX)
C
        IF (HDERR .EQ. 0) THEN
C         no error in read; count the number of station ids
          I = 0
 10       CONTINUE
            I = I + 1
          IF (IVAL(I) .GT. 0 .AND. I .LT. 10) GO TO 10
          TNIDX = I
          IF (IVAL(10) .LE. 0) TNIDX = TNIDX - 1
          IF (CSTA .NE. 'S') THEN
C           First record must start with STATION and contain
C           station identification numbers
            SGRP = 71
            CALL PRNTXT (MESSFL,SCLU,SGRP)
            FLAG = 1
          END IF
C
 20       CONTINUE
C           loop until correct station ids entered
C
C           available stations, put in status block
            C78 = ' '
            C78(1:24) = AVST
            CALL ZSTADD (L1, C78)
            C78 = ' '
            C78(1:72) = STR(23:94)
            CALL ZSTADD (L2, STR(23:94))
            IF (K .LT. 9*TNIDX) THEN
C             write rest of stations
              C78 = ' '
              C78(1:18) = STR(95:112)
              CALL ZSTADD (L3, STR(95:112))
            END IF
C 
C           Number of index sites to use and station ids
            SGRP = 73
            CALL Q1INIT (MESSFL, SCLU, SGRP)
            NIDX = TNIDX
C           max to choose is 6.
            IF (NIDX .GT. 6) NIDX = 6
            CALL QSETIB (L1,L1,NIDX)
            CALL Q1EDIT (IRET)
            CALL QGETI (L1, NIDX)
            CALL QGETIB (NIDX, L2, IDXSTA)
            K = 0
            DO 24 I = 1,NIDX
              DO 22 J = 1,TNIDX
                IF (IDXSTA(I) .EQ. IVAL(J)) THEN
                  K = K + 1
                  ORDER(K) = J
                END IF
 22           CONTINUE
 24         CONTINUE
            FLG = 0
            IF (K .NE. NIDX) THEN
C             one of station numbers not ok
              SGRP = 78
              CALL PRNTXT(MESSFL,SCLU,SGRP)
              FLG = 1
            END IF
          IF (FLG .EQ. 1) GO TO 20
C
C         get data set numbers of index sites
C         looking for data sets for selected index stations
          SGRP = 81
          CALL PRNTXT (MESSFL, SCLU, SGRP)
          DO 25 I = 1,NIDX
            K = ORDER(I)
            IDXSTA(I) = IVAL(K)
 25       CONTINUE
          CALL GTDSNS (WDMSFL,MESSFL,SCLU,NSTAT,NIDX,
     I                 IDXSTA,NDAY,
     O                 TSTYPE,DSNI,FLAG)
C
          IF (FLAG .NE. 0) THEN
C           Could not find all index stations on WDM file
C           so can't procede.  Try again without missing
C           station or add the station to your WDM file.
            SGRP = 82
            CALL PRNTXT (MESSFL, SCLU, SGRP)
          ELSE
C           Read the data (year,mo,day,partial,index)
            FLAG = 0
            I = 1
            IBAD = 0
 30         CONTINUE
C             read the file
              READ (INUNIT,1000,END=40,ERR=50) (DATE(K),K=1,3), 
     &                                  YPR(I),(PX(J),J=1,TNIDX)
              IF (DATE(1).LT.1800 .OR. DATE(1).GT.2010) THEN
                IBAD = IBAD + 1
              ELSE
                DO 35 J = 1,NIDX
                  K = ORDER(J)
                  XIS(I,J) = PX(K)
 35             CONTINUE
                IYR(I) = DATE(1)
                I = I + 1
                IF (I .GT. 100) THEN
C                 only 100 points possible
                  SGRP = 37
                  CALL PRNTXT (MESSFL,SCLU,SGRP)
                END IF
              END IF
            IF (I .LE. 100) GO TO 30
 40         CONTINUE
C
            IF (IBAD .GT. 0) THEN
C             & records had invalid years
              SGRP = 65
              CALL PRNTXI (MESSFL, SCLU, SGRP, IBAD)
            END IF
C           set number of observations read and tell user
            NOBS = I - 1
            IF (NOBS .GE. 4) THEN
              SGRP = 38
              CALL PRNTXI (MESSFL, SCLU, SGRP, NOBS)
            ELSE
              SGRP = 39
              CALL PRNTXT (MESSFL, SCLU, SGRP)
              FLAG = 1
              NOBS = 0
            END IF
C
C         end if (flag .ne. 0)
          END IF
C 
          GOTO 99
C
 50       CONTINUE
C           Error reading record &.
            I = I + 1
            SGRP = 36
            CALL PRNTXI (MESSFL,SCLU,SGRP,I)
            CALL ZMNSST
            IF (I .GT. 4) THEN
C             Do you want to continue(1-y,2-n)
              SGRP = 66
              CALL QRESP (MESSFL,SCLU,SGRP,RESP)
              IF (RESP .EQ. 1) THEN
                NOBS = I - 1
              ELSE
                NOBS = 0
              END IF
            ELSE
C             not enough values to continue
              NOBS = 0
            END IF
C
 99       CONTINUE
        ELSE
C         error in header record
          SGRP = 67
          CALL PRNTXT (MESSFL, SCLU, SGRP)
          NOBS = 0
C       end if (hderr .eq. 0) 
        END IF
C
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   GTDSNS
     I                   (WDMSFL, MESSFL, SCLU, NSTAT,
     I                    NIDX, IDXSTA, NDAY,
     O                    TSTYPE, DSNI, RETCOD)
C
C     + + + PURPOSE + + +
C     This routine finds time series data set numbers given station
C     id and TSTYPE for n-day annual series.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   WDMSFL, MESSFL, NSTAT, DSNI(6,12), NIDX,
     &          RETCOD, IDXSTA(NIDX), NDAY(NSTAT)
      CHARACTER*1   TSTYPE(4,12)
C
C     + + + ARGUMENT DEFINITION + + +
C     WDMSFL  -
C     MESSFL -
C     SCLU -
C     NSTAT  -
C     NIDX   -
C     IDXSTA -
C     NDAY   -
C     TSTYPE -
C     DSNI   -
C     RETCOD -
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   DTYPE, TU, TS, DSMAX, I, N, SGRP, SCLU, LEN, JUST,
     &          OLEN, KOUNT, IVAL(2), II, ML, SF, DSNKNT, IWRT
      CHARACTER*1 CSTA(16)
C
C     + + + EXTERNALS + + +
      EXTERNAL   INTCHR, PRNTXT, INDSFN, PRNTXI, PMXTXI
C
C     + + + DATA INITIALIZATION + + +
      DATA CSTA/16*' '/
C
C     + + + END SPECIFICATIONS + + +
C
      ML = 10
      SF = 1
      DTYPE = 1
      TS = 1
      TU = 6
      KOUNT = 0
      LEN = 3
C     determine TSTYPE
      DO 10 N = 1,NSTAT
        TSTYPE(1,N) = 'L'
        JUST = 0
        CALL INTCHR (NDAY(N),LEN,JUST,OLEN,TSTYPE(2,N))
        IF (TSTYPE(2,N) .EQ. ' ') TSTYPE(2,N) = '0'
        IF (TSTYPE(3,N) .EQ. ' ') TSTYPE(3,N) = '0'
 10   CONTINUE
C
      DO 50 I = 1,NIDX
C       for index stations for each n-day statistic
        DO 40 N = 1,NSTAT
C         find data set number
          DSMAX = 1
          CALL INDSFN (WDMSFL, DTYPE, TSTYPE(1,N), CSTA, IDXSTA(I),
     &                 TS, TU, DSMAX, DSNI(I,N), DSNKNT)
          IF (DSNKNT .EQ. 0) THEN
C           data set for index station not found
            SGRP = 75
            CALL PRNTXI (MESSFL, SCLU, SGRP, IDXSTA(I))
            DSNI(I,N) = 0
            KOUNT = KOUNT + 1
          ELSE
C           Data set & found for station &.
            IVAL(1) = DSNI(I,N)
            IVAL(2) = IDXSTA(I)
            II = 2
            IWRT = 0
            SGRP = 83
            CALL PMXTXI (MESSFL,SCLU,SGRP,ML,SF,IWRT,II,IVAL)
          END IF
 40     CONTINUE
 50   CONTINUE
C
      RETCOD = KOUNT
C
      RETURN
      END
C
C
C
      SUBROUTINE   PSTANL
     I                  (MESSFL,SCLU,OFILE,NOBS,NIDX,IDXSTA,
     M                   YPR,XIS,
     O                   B,SE,XTXINV,RESID,COOKD,PRED,FLAG)
C
C     + + + PURPOSE + + +
C     This routine performs ordinary least squares on data necessary for
C     the Stedinger-Thomas method to estimate low flow statistics for
C     the partial record site.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   MESSFL,SCLU,OFILE,NOBS,NIDX,IDXSTA(NIDX),FLAG
      REAL   PRED(NOBS),RESID(NOBS),COOKD(NOBS),YPR(100),XIS(100,6)
      DOUBLE PRECISION   B(10,1),SE,XTXINV(10,10)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MESSFL -
C     SCLU   -
C     OFILE  -
C     NOBS   -
C     NIDX   -
C     IDXSTA -
C     YPR    -
C     XIS    -
C     B      -
C     SE     -
C     XTXINV -
C     RESID  -
C     COOKD  -
C     PRED   -
C     FLAG   -
C
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      SGRP
      CHARACTER*48 WNDNAM
C
C     + + + EXTERNALS + + +
      EXTERNAL   LOGTRN, PRNTXT, LREG6, ZWNSET
C
C     + + + END SPECIFICATIONS + + +
C     take logs
      CALL LOGTRN (NOBS, NIDX,  YPR, XIS, FLAG)
C
      IF (FLAG .EQ. 1) THEN
C       Can't procede because of zeros in data.  Add a
C       constant to all values or eliminate the zero points.
        SGRP = 69
        CALL PRNTXT (MESSFL,SCLU,SGRP)
      END IF
C
      IF (FLAG .EQ. 0) THEN
C       set window name for next screens
        WNDNAM = 'Analyze (PASA)'
        CALL ZWNSET (WNDNAM)
C       compute statistics
        CALL LREG6 (OFILE, NOBS, NIDX, IDXSTA, XIS, YPR, 
     &              B,SE,XTXINV,RESID,COOKD,PRED,FLAG)
        IF (FLAG .EQ. 1) THEN
C         too few points to analyze, need more concurrent dates
          SGRP = 13
          CALL PRNTXT (MESSFL, SCLU, SGRP)
        END IF
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   LOGTRN
     I                   (NPTS, NIDX,
     M                    Y, X,
     O                    ERRFLG)
C
C     + + + PURPOSE + + +
C     The purpose of this routine is to check with user to make a
C     log transformation and do the transformations if the data
C     has no 0.0 or negative numbers.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   NPTS, NIDX, ERRFLG
      REAL      Y(100), X(100,6)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NPTS   -
C     NIDX  -
C     Y      -
C     X      -
C     AXTYP  -
C     ERRFLG -
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   N, J
C
C     + + + INTRINSICS + + +
      INTRINSIC   ALOG10
C
C     + + + EXTERNALS + + +
      EXTERNAL   PRNTXT
C
C     + + + END SPECIFICATIONS + + +
C
      ERRFLG = 0
C
C     Take logs
      DO 50 N = 1,NPTS
        IF (Y(N) .LT. 1.0E-9) ERRFLG = 1
        DO 40 J = 1,NIDX
          IF (X(N,J) .LT. 1.0E-9) ERRFLG = 1
 40     CONTINUE
 50   CONTINUE
C
      IF (ERRFLG .EQ. 0) THEN
C       ok, take logs
        DO 60 N = 1,NPTS
          Y(N) = ALOG10(Y(N))
          DO 55 J = 1,NIDX
            X(N,J) = ALOG10(X(N,J))
 55       CONTINUE
 60     CONTINUE
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   LREG6
     I                (OFILE, NOBS, NIDX , IDXSTA, W, Y, 
     O                 B, SE, XTXINV, RESID, COOKD, PRED, RETCOD)
C
C     + + + PURPOSE + + +
C     This routine does least squares regression on up to
C     six explanatory variables, print results on a file and returns
C     the predicted values, cook D  and residuals for plotting.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   NOBS, NIDX , OFILE, RETCOD, IDXSTA(NIDX)
      REAL    W(100,6), Y(100), PRED(NOBS),RESID(NOBS),COOKD(NOBS)
      DOUBLE PRECISION   B(10,1), SE, XTXINV(10,10)
      CHARACTER*8    CON
C
C     + + + ARGUMENT DEFINITIONS + + +
C     OFILE  - Fortran unit number of output file
C     NOBS   - number of observations
C     NIDX   - number of index stations (max is 6)
C     W      - (nobsxnstat)matrix of data for index stations
C     Y      - response variable
C     B      - regression coefficients
C     SE     - standard error of estimate
C     XTXINV - inverse of XTX matrix
C     RESID  - residual
C     COOKD  - Cook's D statistic
C     PRED   - predicted values
C     RETCOD - 1 if too few points for analysis, else 0
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      NCOLS, I, J, IDF, ID1, L78, L1, IWRT, INIT, DONEFG
      REAL         SB, T, TABS, PVAL
      CHARACTER*1  BUFF(78), BLNK
      CHARACTER*80 CARD
      DOUBLE PRECISION   X(100,10),XB(100,1),Z(100,1),
     &                   XT(10,100),XTZ(10,1),BT(1,10),ZT(1,100),
     &                   T1(1,1),T2(1,1),H(100,100),XTXXT(10,100),
     &                   XTX(10,10),R,VAR,DET,SUM,YBAR,SSREG,SSTOT,
     &                   FRATIO,RADJ,SRESID,HII,SSRES,REGMS
C
C     + + + INTRINSICS + + +
      INTRINSIC   SQRT, SNGL, FLOAT, ABS, DBLE, DSQRT
C
C     + + + FUNCTIONS + + +
      REAL   STUTP
C
C     + + + EXTERNALS + + +
      EXTERNAL   MULTPY, INVERT, STUTP, ZBLDWR, CVARAR, QRESP
C
C     + + + OUTPUT FORMATS + + +
 1998 FORMAT(' ANOVA TABLE')
 1999 FORMAT(                 ' SOURCE',T15,'DF',T25,'SS',T40,'MS',
     *T55,'F',T62,'R SQ   ADJ. R SQ')
 2001 FORMAT(' REGRESSION',I5,2G15.5,3F10.2)
 2002 FORMAT(' RESIDUAL  ',I5,2G15.5)
 2003 FORMAT(' TOTAL     ',I5,G15.5)
 2008 FORMAT(' REGRESSION COEFFICIENTS')
 2009 FORMAT(                                 '  VARIABLE       COEFFICI
     1ENT   STANDARD ERROR   T FOR H0:BETA=0','    PROB>|T|')
 2010 FORMAT(A80)
 2020 FORMAT(2X,I8,3G18.5,F10.4)
 2021 FORMAT(2X,A8,3G18.5,F10.4)
 2050 FORMAT(1X,I3,4G13.5,2F10.3)
 2051 FORMAT(   ' RESIDUAL ANALYSIS')
 2052 FORMAT(' NO.    OBSERVED    PREDICTED',T35,'RESIDUAL',T48,
     &'STANDARD R',T61,'LEVERAGE',T71,'COOK D')
 2100 FORMAT(A)
C
C     + + + END SPECIFICATIONS + + +
C
      L78 = 78
      L1 = 1
      BLNK = ' '
C
C     resid=residual
C     sresid=standardized residual
C     hii=leverage
C     t=t statistic
C     pval=p level
C
C     Load matrices
C
      NCOLS=NIDX +1
      SUM=0.0
      DO 10 I=1,NOBS
         X(I,1)=1.0
         X(I,2)=DBLE(W(I,1))
         Z(I,1)=DBLE(Y(I))
         ZT(1,I)=Z(I,1)
         SUM=SUM+Z(I,1)
         IF(NCOLS.GT.2)X(I,3)=DBLE(W(I,2))
         IF(NCOLS.GT.3)X(I,4)=DBLE(W(I,3))
         IF(NCOLS.GT.4)X(I,5)=DBLE(W(I,4))
         IF(NCOLS.GT.5)X(I,6)=DBLE(W(I,5))
         IF(NCOLS.GT.6)X(I,7)=DBLE(W(I,6))
 10   CONTINUE
      YBAR=SUM/NOBS
C
C     Transpose x
C
      DO 20 I=1,NOBS
        DO 15 J=1,NCOLS
          XT(J,I)=X(I,J)
 15     CONTINUE
 20   CONTINUE
C
C     Compute b and XTXinv
C
      CALL MULTPY(XT,X,NCOLS,NOBS,NCOLS,10,10,100,XTX)
      CALL INVERT(NCOLS,10,XTX,0,OFILE,DET,XTXINV)
      CALL MULTPY(XT,Z,NCOLS,NOBS,1,10,10,100,XTZ)
      CALL MULTPY(XTXINV,XTZ,NCOLS,NCOLS,1,10,10,10,B)
C
C     Compute ANOVA table
C
      DO 30 I=1,NCOLS
        BT(1,I)=B(I,1)
 30   CONTINUE
      CALL MULTPY(BT,XTZ,1,NCOLS,1,1,1,10,T1)
      CALL MULTPY(ZT,Z,1,NOBS,1,1,1,100,T2)
      SSREG=T1(1,1)-NOBS*YBAR**2
      SSTOT=T2(1,1)-NOBS*YBAR**2
      SSRES=T2(1,1)-T1(1,1)
      IDF=NOBS-NCOLS
      ID1=NOBS-1
      IF(IDF.GE.1)THEN
        VAR=SSRES/IDF
        SE=DSQRT(VAR)
        R=SSREG/SSTOT
        RADJ=1.0-(FLOAT(NOBS-1)/FLOAT(NOBS-NCOLS))*(1.0-R)
        REGMS=SSREG/NIDX
        FRATIO=REGMS/VAR
C
C       Write ANOVA table to terminal and file
C
C       header
        WRITE(CARD,1998)
        WRITE(OFILE,2010) CARD
        CALL CVARAR (L78,CARD,L78,BUFF)
        INIT = 1
        IWRT = 1
        CALL ZBLDWR (L78,BUFF,INIT,IWRT,DONEFG)
        WRITE(CARD,1999)
        WRITE(OFILE,2010) CARD
        WRITE(OFILE,2100) BLNK
        CALL CVARAR (L78,CARD,L78,BUFF)
        INIT = 0
        CALL ZBLDWR (L78,BUFF,INIT,IWRT,DONEFG)
C       regression results
        WRITE(CARD,2001)NIDX , SSREG, REGMS, FRATIO, R,RADJ
        WRITE(OFILE,2010) CARD
        CALL CVARAR (L78,CARD,L78,BUFF)
        CALL ZBLDWR (L78,BUFF,INIT,IWRT,DONEFG)
C       residual results
        WRITE(CARD,2002)IDF,SSRES,VAR
        WRITE(OFILE,2010) CARD
        CALL CVARAR (L78,CARD,L78,BUFF)
        CALL ZBLDWR (L78,BUFF,INIT,IWRT,DONEFG)
C       totals
        WRITE(CARD,2003)ID1, SSTOT
        WRITE(OFILE,2010) CARD
        CALL CVARAR (L78,CARD,L78,BUFF)
        IWRT = 0
        CALL ZBLDWR (L78,BUFF,INIT,IWRT,DONEFG)
C
C       Regression coefficients table
C
C       header
        WRITE(CARD,2008)
        WRITE(OFILE,2100) BLNK
        WRITE(OFILE,2100) BLNK
        WRITE(OFILE,2100) BLNK
        WRITE(OFILE,2010) CARD
        CALL CVARAR (L78,CARD,L78,BUFF)
        INIT = 1
        IWRT = 1
        CALL ZBLDWR (L78,BUFF,INIT,IWRT,DONEFG)
        WRITE(CARD,2009)
        WRITE(OFILE,2010) CARD
        CALL CVARAR (L78,CARD,L78,BUFF)
        INIT = 0
        CALL ZBLDWR (L78,BUFF,INIT,IWRT,DONEFG)
        WRITE(OFILE,2100) BLNK
        SB=SNGL(XTXINV(1,1)*SE**2)
        SB=SQRT(SB)
        T=SNGL(B(1,1))/SB
        TABS=ABS(T)
        PVAL=2.0*STUTP(-TABS,IDF)
        IF(PVAL.LT..0001)PVAL=.0001
        CON='CONSTANT'
C       results
        WRITE(CARD,2021)CON,B(1,1),SB,T,PVAL
        WRITE(OFILE,2010) CARD
        CALL CVARAR (L78,CARD,L78,BUFF)
        CALL ZBLDWR (L78,BUFF,INIT,IWRT,DONEFG)
        DO 40 I=2,NCOLS
          SB=SNGL(XTXINV(I,I)*SE**2)
          SB=SQRT(SB)
          T=SNGL(B(I,1))/SB
          TABS=ABS(T)
          PVAL=2.0*STUTP(-TABS,IDF)
          IF(PVAL.LT..0001)PVAL=.0001
          WRITE(CARD,2020)IDXSTA(I-1),B(I,1),SB,T,PVAL
          WRITE(OFILE,2010) CARD
          CALL CVARAR (L78,CARD,L78,BUFF)
          CALL ZBLDWR (L78,BUFF,INIT,IWRT,DONEFG)
 40     CONTINUE
C       hold for user acceptance
        IWRT = 0
        CALL ZBLDWR (L1,BLNK,INIT,IWRT,DONEFG)
C
C       Residual analysis
C
        CALL MULTPY(XTXINV,XT,NCOLS,NCOLS,NOBS,10,10,10,XTXXT)
        CALL MULTPY(X,XTXXT,NOBS,NCOLS,NOBS,100,100,10,H)
        CALL MULTPY(X,B,NOBS,NCOLS,1,100,100,10,XB)
C
C       header
        WRITE(OFILE,2100) BLNK
        WRITE(OFILE,2100) BLNK
        WRITE(CARD,2051)
        WRITE(OFILE,2010) CARD
        CALL CVARAR (L78,CARD,L78,BUFF)
        INIT = 1
        IWRT = 1
        CALL ZBLDWR (L78,BUFF,INIT,IWRT,DONEFG)
        WRITE(CARD,2052)
        WRITE(OFILE,2010) CARD
        CALL CVARAR (L78,CARD,L78,BUFF)
        INIT = 0
        CALL ZBLDWR (L78,BUFF,INIT,IWRT,DONEFG)
        DO 60 I=1,NOBS
          RESID(I)= SNGL(Z(I,1)- XB(I,1))
          PRED(I)= SNGL(XB(I,1))
          HII=H(I,I)
          SRESID=DBLE(RESID(I))/(SE*DSQRT(1.0-HII))
          COOKD(I)=SNGL(SRESID**2*HII/((1.0-HII)*NCOLS))
          WRITE(CARD,2050)I,Z(I,1),XB(I,1),RESID(I),SRESID,HII,COOKD(I)
          WRITE(OFILE,2010) CARD
          IF (DONEFG .NE. 7) THEN
            IF (MOD(I,8) .EQ. 0) IWRT = 0
            IF (MOD(I-1,8) .EQ. 0) THEN
              IWRT = 1
              INIT = 1
            END IF
            IF (MOD(I-2,8) .EQ. 0) INIT = 0
            CALL CVARAR (L78,CARD,L78,BUFF)
            CALL ZBLDWR (L78,BUFF,INIT,IWRT,DONEFG)
          END IF
 60     CONTINUE
        IF (IWRT .EQ. 1) THEN
C         hold for user response if not done after last write
          IWRT = 0
          CALL ZBLDWR (L1,BLNK,INIT,IWRT,DONEFG)
        END IF
        RETCOD = 0
C
      ELSE
C       not enough points
        RETCOD = 1
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   PSTPLT
     I                   (MESSFL, SCLU, NPTS, YPR, PRED, RESID, COOKD)
C
C     + + + PURPOSE + + +
C     This routine sets up arrays to plot results of the least squares
C     analysis and calls the routine that set ??? and makes the plot.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER      MESSFL, SCLU, NPTS
      REAL         PRED, RESID, COOKD, YPR(100)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MESSFL - Fortran unit number of message file
C     SCLU   - screen cluster number on message file
C     NPTS   -
C     YPR    -
C     PRED   -
C     RESID  -
C     COOKD  -
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      I, J, SGRP, XOPT, YOPT, IRET, CNUM, CBAS
      REAL         Y(100), X(100)
      CHARACTER*1  TITL(9,4)
C
C     + + + EXTERNALS + + +
      EXTERNAL   QRESPM, COPYR, PLTVRL
      EXTERNAL   Q1INIT, Q1EDIT, QGTCOB
C
C     + + + DATA INITIALIZATIONS + + +
      DATA  TITL/'M','e','a','s','u','r','e','d',' ',
     &           'P','r','e','d','i','c','t','e','d',
     &           'R','e','s','i','d','u','a','l',' ',
     &           'C','o','o','k',' ','D',' ',' ',' '/
C
C     + + + END SPECIFICATIONS + + +
C
 10   CONTINUE
C       get variable for y-axis & x-axis
        SGRP = 100
        CALL Q1INIT (MESSFL, SCLU, SGRP)
        CALL Q1EDIT (IRET)
C     if user chose 'Oops', redraw screen
      IF (IRET .EQ. -1) GOTO 10
C
C     retrieve user input
      CNUM = 1
      CBAS = 1
      CALL QGTCOB (CNUM, CBAS, 
     O             XOPT)
      CBAS = 2
      CALL QGTCOB (CNUM, CBAS,
     O             YOPT)
C
      IF (YOPT .EQ. 1) THEN
        CALL COPYR (NPTS,YPR,Y)
      ELSE IF (YOPT .EQ. 2) THEN
        CALL COPYR (NPTS,PRED,Y)
      ELSE IF (YOPT .EQ. 3) THEN
        CALL COPYR (NPTS,RESID,Y)
      ELSE IF (YOPT .EQ. 4) THEN
        CALL COPYR (NPTS,COOKD,Y)
      END IF
C
      IF (XOPT .EQ. 1) THEN
        CALL COPYR (NPTS,YPR,X)
      ELSE IF (XOPT .EQ. 2) THEN
        CALL COPYR (NPTS,PRED,X)
      ELSE IF (XOPT .EQ. 3) THEN
        CALL COPYR (NPTS,RESID,X)
      ELSE IF (XOPT .EQ. 4) THEN
        CALL COPYR (NPTS,COOKD,X)
      END IF
      I = YOPT
      J = XOPT
      CALL PLTVRL  (MESSFL, SCLU, NPTS, Y, X, TITL(1,I), TITL(1,J))
C
      RETURN
      END
C
C
C
      SUBROUTINE   PLTVRL
     I                   (MESSFL, SCLU, MAXP, YPT, XPT, YTITL, XTITL)
C
C     + + + PURPOSE + + +
C     calls routines to make a plot of the results
C     from the Thomas-Stedinger technique.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER      MESSFL, SCLU, MAXP
      REAL         YPT(MAXP), XPT(MAXP)
      CHARACTER*1  XTITL(9), YTITL(9)
C
C     + + + ARGUMENT DEFINITION + + +
C     MAXP   - number of points in YPT and XPT to plot
C     YPT    - array of Y values to plot
C     XPT    - array of X values to plot
C     YTITL  - y label for plot
C     XTITL  - x label for plot
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     I, I1, L9, L40, L80, SGRP, IPOS, RETCOD,
     $            GWHICH(2), GTICS(4), GBVALF(4), GCTYP(2), GCOLOR(2), 
     $            ICHK(7), GPATRN(2), GTRANF(2), GXTYP, 
     $            GYTYP(2), GLNTYP(2), GSYMBL(2), AXTYP, IVAR, NUMCRV, 
     $            NUMVAR, ICRV, YVAR, XVAR, WSID, ICLOS, IWAIT, MORE,
     $            WNDFLG, IND, DEVTYP, DEVCOD
      REAL        SIZEL, XPAGE, YPAGE, XPHYS, YPHYS, XLEN, YLEN, ALEN, 
     $            GVMIN(2), GVMAX(2), PLMN(4), PLMX(4)
      CHARACTER*1 OLAB(20,2), BLNK, XLAB(80), YLAB(80), YXLAB(80),
     $            CLAB1(20,2), TITL1(240)
      CHARACTER*8 WNAM(2)
C
C     + + + EXTERNALS + + +
      EXTERNAL    GPINIT, GPNCRV, GPDATR, GPCURV, GPVAR, SCALIT, GPSCLE
      EXTERNAL    GPLABL, GPSIZE, GPWCXY, PSTUPW, PLTONE, PDNPLT, PROPLT
      EXTERNAL    GPDEVC, GGDVTY, GPLBXB
      EXTERNAL    ZIPI, ZIPC, CHRCHR
C
C     + + + DATA INITIALIZATIONS + + +
      DATA  SIZEL, XPAGE, YPAGE, XPHYS, YPHYS, XLEN, YLEN, ALEN, AXTYP
     &     / 0.12,  10.0,  8.0,    1.5,   1.5,  7.5,  5.0,  0.0,     1/
      DATA  BLNK /' '/
      DATA  L9, L40, L80
     $     / 9,  40,  80/
      DATA  TITL1 /240*' '/
      DATA  OLAB /'Y','-','a','x','i','s',14*' ',
     $            'X','-','a','x','i','s',14*' '/
C
C     + + + END SPECIFICATIONS + + +
C
      I1 = 1
      I = 7
      CALL ZIPI (I, I1, ICHK)
C     don't allow modification of axes
      ICHK(2) = 2
      WNAM(1) = 'Modify'
      WNAM(2) = 'PASPlM'
C     initialize graphics buffer
      CALL GPINIT
C     set plotting device to screen
      DEVTYP = 1
      IND = 40
      CALL ANPRGT (IND,DEVCOD)
      CALL GPDEVC (DEVTYP,DEVCOD)
C
C     set legend to blanks
      CALL ZIPC (L40, BLNK, CLAB1)
C     assign axes labels
      CALL ZIPC (L80, BLNK, XLAB)
      CALL ZIPC (L80, BLNK, YLAB)
      CALL CHRCHR (L9, XTITL, XLAB)
      CALL CHRCHR (L9, YTITL, YLAB)
C
C     number of curves and variables
      NUMCRV = 1
      NUMVAR = 2
      CALL GPNCRV (NUMCRV, NUMVAR)
C
C     save the Y data
      IVAR = 1
      IPOS= 1
      CALL GPDATR (IVAR, IPOS, MAXP, YPT, RETCOD)
C     save the X data
      IVAR = 2
      IPOS= IPOS+ MAXP
      CALL GPDATR (IVAR, IPOS, MAXP, XPT, RETCOD)
C
C     save curve info for xy plot
      GCTYP(1) = 6
C     set line type to none
      GLNTYP(1) = 0
C     set symbol type to '+'
      GSYMBL(1) = 2
C     black/white color
      GCOLOR(1)= 1
C     no pattern
      GPATRN(1)= 0
      CALL GPCURV (GCTYP, GLNTYP, GSYMBL, GCOLOR, GPATRN, CLAB1)
C
C     save data ranges
      GWHICH(1)= 1
      GWHICH(2)= 4
      GVMIN(1) = 1.0E30
      GVMAX(1) = -1.0E30
      GVMIN(2) = 1.0E30
      GVMAX(2) = -1.0E30
      DO 10 I= 1, MAXP
        IF (YPT(I).GT.GVMAX(1)) GVMAX(1)= YPT(I)
        IF (YPT(I).LT.GVMIN(1)) GVMIN(1)= YPT(I)
        IF (XPT(I).GT.GVMAX(2)) GVMAX(2)= XPT(I)
        IF (XPT(I).LT.GVMIN(2)) GVMIN(2)= XPT(I)
 10   CONTINUE
C     no transformation (data was already log-transformed)
      GTRANF(1) = 1
      GTRANF(2) = 1
      CALL GPVAR (GVMIN, GVMAX, GWHICH, GTRANF, OLAB)
C
C     assign variables to first (only) curve
      ICRV = 1
      YVAR = 1
      XVAR = 2
      CALL GPWCXY (ICRV, YVAR, XVAR)
C
C     scale data, Y then X
      CALL SCALIT (AXTYP, GVMIN(1), GVMAX(1), 
     O             PLMN(1), PLMX(1))
      CALL SCALIT (AXTYP, GVMIN(2), GVMAX(2), 
     O             PLMN(4), PLMX(4))
C
C     set up axis scaling
      GTICS(1) = 10
      GTICS(4) = 10
      CALL ZIPI (4, 1, GBVALF)
      CALL GPSCLE (PLMN, PLMX, GTICS, GBVALF)
C     define axes
      GXTYP = AXTYP
      GYTYP(1) = AXTYP
      GYTYP(2) = 0
      CALL GPLABL (GXTYP, GYTYP, ALEN, YLAB, YXLAB, CLAB1, TITL1)
C     assign x-axis label
      CALL GPLBXB (XLAB)
C     set up sizes
      CALL GPSIZE (SIZEL, XPAGE, YPAGE, XPHYS, YPHYS, XLEN, YLEN, ALEN)
C
C     make the plot
      WSID = DEVTYP
      ICLOS = 0
      IWAIT = 0
      CALL PSTUPW (WSID, RETCOD)
      CALL PLTONE
      CALL PDNPLT (WSID, ICLOS, IWAIT)
C
 20   CONTINUE
C       allow user to modify plot:  replot, modify, done
        SGRP = 101
        CALL QRESP (MESSFL, SCLU, SGRP, MORE)
        IF (MORE .EQ. 1) THEN
C         replot
          WSID = DEVTYP
          ICLOS = 0
          CALL PSTUPW (WSID, RETCOD)
          CALL PLTONE
          CALL PDNPLT (WSID, ICLOS, IWAIT)
        ELSE IF (MORE .EQ. 2) THEN
C         modify specs
          CALL PROPLT (MESSFL, ICHK, WNAM,
     O                 WNDFLG)
          IF (WNDFLG .EQ. 1) THEN
C           user changed device; close the current device
            ICLOS = 1
            CALL PDNPLT (DEVTYP, ICLOS, IWAIT)
C           get new device type
            CALL GGDVTY (DEVTYP)
          END IF
        END IF
      IF (MORE .LE. 2) GO TO 20
C
      RETURN
      END
C
C
C
      SUBROUTINE   PSTCOM
     I                  (MESSFL, SCLU, WDMSFL, OFILE, NSTAT, NIDX,
     I                   DSNI, DSNP, B, SE, XTXINV)
C
C     + + + PURPOSE + + +
C     This routine does Stedinger-Thomas calculations and puts attributes
C     on partial record data sets.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   MESSFL, SCLU, WDMSFL, NSTAT, NIDX, OFILE
      INTEGER   DSNP(NSTAT), DSNI(1,NSTAT)
      DOUBLE PRECISION   B, SE, XTXINV
C
C     + + + ARGUMENT DEFINITIONS + + +
C
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   SGRP,N,RETCOD,NY
      REAL   Z(100,6)
C
C     + + + EXTERNALS + + +
      EXTERNAL   PRNTXI, GETZZS, THOMAS
C
C     + + + END SPECIFICATIONS + + +
C     begin loop to compute and store attributes for each
C     low flow statistic
      DO 70 N = 1,NSTAT
C       For partial record data set &:
        SGRP = 87
        CALL PRNTXI (MESSFL,SCLU,SGRP,DSNP(N))
C       get annual series for index sites
C       take logs, drop zeros, only for concurrent
        CALL GETZZS (WDMSFL, DSNI(1,N), NIDX, NY, Z)
C
        CALL THOMAS (OFILE, MESSFL, SCLU, WDMSFL, DSNP(N),
     &               NY, NIDX, Z, B, SE, XTXINV, RETCOD)
 70   CONTINUE
C
      RETURN
      END
C
C
C
      SUBROUTINE   GETZZS
     I                   (WDMSFL, DSNI, NSTAT,
     O                    NY, Z)
C
C     + + + PURPOSE + + +
C     This routine gets annual series for index sites for a common
C     period of non-zero flows and transforms data to logs base 10.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   WDMSFL, DSNI(6), NSTAT,NY
      REAL      Z(100,6)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     WDMSFL  - Fortran unit number of users message file
C     DSNI   - data set numbers of index sites
C     NSTAT  - number of index sites
C     NY     - number of non-zero concurrent years
C     Z      - log of flows for concurrent period
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I, CNT, N, FLG, GPFLG, TDSFRC, QFLG, DTRAN, L1, L2,
     &          EDAT(6,6), SDAT(6,6), EDATE(6), SDATE(6), TS, TU,
     &          RETCOD
C
C     + + + INTRINSICS + + +
      INTRINSIC   ALOG10
C
C     + + + EXTERNALS + + +
      EXTERNAL   WDTGET, TIMDIF, DLIMIT, WTFNDT
C
C     + + + END SPECIFICATIONS + + +
C
      TS = 1
      TU = 6
      DTRAN = 0
      QFLG = 30
      GPFLG = 0
      TDSFRC = 0
      DO 20 I = 1,NSTAT
C       get start and end dates for each data set
        CALL WTFNDT(WDMSFL,DSNI(I),GPFLG,TDSFRC,SDAT(1,I),EDAT(1,I),
     &              RETCOD)
 20   CONTINUE
C
C     find common period
      L1 = 1
      L2 = 2
      CALL DLIMIT (SDAT,NSTAT,L2,SDATE)
      CALL DLIMIT (EDAT,NSTAT,L1,EDATE)
      CALL TIMDIF (SDATE,EDATE,TU,TS,NY)
C
      IF (NY .GT. 0) THEN
C
C       get the data
        DO 30 I = 1,NSTAT
          CALL WDTGET (WDMSFL,DSNI(I),TS,SDATE,NY,DTRAN,QFLG,TU,
     &                 Z(1,I),RETCOD)
 30     CONTINUE
C
C       screen for zeros and take logs
        CNT = 0
        DO 50 N = 1,NY
          FLG = 0
          DO 45 I = 1,NSTAT
            IF (Z(N,I) .LE. 0.0) FLG = 1
 45       CONTINUE
          IF (FLG .EQ. 0) THEN
C           ok year
            CNT = CNT + 1
            DO 40 I = 1,NSTAT
              Z(CNT,I) = ALOG10(Z(N,I))
 40         CONTINUE
          END IF
 50     CONTINUE
C
        NY = CNT
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   THOMAS
     I                  (OFILE,MESSFL,SCLU,WDMSFL,DSN,NY,NIDX,
     I                   Z,B,SE,XTXINV,
     O                   RETCOD)
C
C     + + + PURPOSE + + +
C     This subroutine uses the Stedinger-Thomas method to estimate
C     low flow characterictics from base-flow measurements correlated
C     with concurrent data at up to six index gaging stations.  The
C     mean, std dev, skew, equivalent years, and flow statistics
C     are put on the WDM data set.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   NY,NIDX,OFILE,DSN,RETCOD,WDMSFL,MESSFL,SCLU
      REAL   Z(100,6)
      DOUBLE PRECISION   B(10,1),SE,XTXINV(10,10)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     OFILE  - Fortran unit number of output file
C     MESSFL - Fortran unit number of GLSNET message file
C     SCLU   - cluster number for message file
C     WDMSFL - Fortran unit number of users WDM file
C     DSN    - data set number of partial record site
C     NY     - number of concurrent years for index sites
C     NIDX   - number of index stations
C     Z      - (ny x nidx) matrix of annual series for index stations
C     B      - vector of regression coefficient from LREG6
C     SE     - standard error from LREG6
C     XTXINV - inverse of XTX matrix from LREG6
C     RETCOD - return code, 0 if all ok
C
C     + + + LOCAL VARIABLES + + +
      REAL   SKEW(10),R(4),AK(4),AVGSKU,SUM4,P,TEMP
      DOUBLE PRECISION   BT(1,10),X(100,10),COV,VXX(10,10),VXXT(10,10),
     &                   XMU(10,1),XMUT(1,10),YMU(1,1),WORK(10,1),
     &                   T1(1,1),VS(10,10),VSB(10,1),A(10,10),
     &                   XT(10,100),YHAT(4),VYHAT(4),VMU,SUM,SUM2,SUM3,
     &                   SD,S1,TRACE,TERM1,TERM2,YVAR,YSD,HARD,
     &                   EQYRS(4),VARV,DVAL(1),DNY
      INTEGER   NCOLS,J,I,IH,IG,ML,SF,L6,OTYP(2),IDUM,SALEN,SAIND,
     &          SGRP,L2,SATYP,L78,ERRFLG,ERRCOD(4),ISTANO,
     &          RETFLG,IWRT,INIT, DONEFG
      CHARACTER*1   BUFF(80),BLNK,ATTR(6,4),CTYPE(4)
      CHARACTER*48   WNDNAM
      CHARACTER*80   CARD
C
C     + + + INTRINSICS + + +
      INTRINSIC   SNGL, DSQRT, DBLE
C
C     + + + FUNCTIONS + + +
      REAL   WILFRT, GAUSAB
C
C     + + + EXTERNALS + + +
      EXTERNAL   WILFRT, MULTPY, GAUSAB, WDBSAR, PRNTXR, PRNTXI
      EXTERNAL   WDBSGC, CHRCHR, WDBSGX, PMXTXM, ZBLDWR, CVARAR, ZWNSET
      EXTERNAL   WDBSGI
C
C     + + + DATA INITIALIZATIONS + + +
      DATA  ATTR/'L','0','0','0','0','2', 'L','0','0','0','0','5',
     &           'L','0','0','0','1','0', 'L','0','0','0','2','0'/
      DATA L2/2/,  L6/6/,  ML/10/,  SF/1/,  L78/78/,  BLNK/' '/
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT(1X,G15.5,G10.2,3G15.5)
 2001 FORMAT(' ESTIMATED FLOW CHARACTERISTICS AT PARTIAL-RECORD SITE',
     &        I12)
 2002 FORMAT(  ' MEAN=',G13.5)
 2003 FORMAT(                   ' STD DEV=',G13.5)
 2004 FORMAT(                                 ' SKEW COEF.=', G13.5)
 2005 FORMAT(
     &  ' RETURN PERIOD',T20,' Y        ',T30,'  LOG Y ',
     &  T45,'  VARIANCE',
     $T60,'EQUIVALENT YEARS')
 2010 FORMAT(A80)
 2100 FORMAT (A)
C
C     + + + END SPECIFICATIONS + + +
C
C     load data
C
      NCOLS=NIDX +1
      DO 11 I=1,NY
        X(I,1)=1.0
        X(I,2)=DBLE(Z(I,1))
        IF(NCOLS.GT.2)X(I,3)=DBLE(Z(I,2))
        IF(NCOLS.GT.3)X(I,4)=DBLE(Z(I,3))
        IF(NCOLS.GT.4)X(I,5)=DBLE(Z(I,4))
        IF(NCOLS.GT.5)X(I,6)=DBLE(Z(I,5))
        IF(NCOLS.GT.6)X(I,7)=DBLE(Z(I,6))
 11   CONTINUE
C
C     Center data
C
      DNY = DBLE(NY)
      DO 30 J=1,NCOLS
         SUM=0.0
         SUM2=0.0
         SUM3=0.0
         DO 20 I=1,NY
            SUM=SUM+X(I,J)
            SUM2=SUM2+X(I,J)**2
            SUM3=SUM3+X(I,J)**3
 20      CONTINUE
         XMUT(1,J)=SUM/DNY
         XMU(J,1)=SUM/DNY
         IF(J.GT.1)THEN
            SD=DSQRT((SUM2-SUM**2/DNY)/(DNY-1.0))
            S1=SUM3*DNY**2-3.0*DNY*SUM*SUM2+2.0*SUM**3
            SKEW(J-1)=SNGL(S1/(SD**3*DNY*(DNY-1.0)*(DNY-2.0)))
         END IF
         DO 22 I=1,NY
            X(I,J)=X(I,J)-SUM/DNY
 22      CONTINUE
 30   CONTINUE
C
C     Transpose x
C
      DO 40 I=1,NY
        DO 35 J=1,NCOLS
          XT(J,I)=X(I,J)
 35     CONTINUE
 40   CONTINUE
C
C     Compute variance matrices
C
      CALL MULTPY(XT,X,NCOLS,NY,NCOLS,10,10,100,VXX)
      DO 50 I=1,NCOLS
        BT(1,I)=B(I,1)
        DO 45 J=1,NCOLS
          VXX(I,J)=VXX(I,J)/(DNY-1.0)
 45     CONTINUE
 50   CONTINUE
C
C     Compute ymu yvar
C
      CALL MULTPY(XMUT,B,1,NCOLS,1,1,1,10,YMU)
      CALL MULTPY(VXX,B,NCOLS,NCOLS,1,10,10,10,WORK)
      CALL MULTPY(BT,WORK,1,NCOLS,1,1,1,10,T1)
      CALL MULTPY(VXX,XTXINV,NCOLS,NCOLS,NCOLS,10,10,10,VS)
      TRACE=0.0
C     Following comment out because TRACE is greater than 1.0 in
C     some cases with few partial record measurements and small
C     variance for those mesaurements.  Also it is not supposed
C     to be significant for large data sets.
C     DO 55 I=1,NCOLS
C       TRACE=TRACE+VS(I,I)
C55   CONTINUE
      TERM2=SE**2*(1.0-TRACE)
      YVAR=TERM2+T1(1,1)
C
C     Compute k values
C
      SUM4=0.0
      DO 60 I=1,NIDX
        SUM4=SUM4+SKEW(I)
 60   CONTINUE
      AVGSKU=SUM4/NIDX
      R(1)=2.0
      R(2)=5.0
      R(3)=10.0
      R(4)=20.0
C
C     Compute var
C
      CALL MULTPY(XTXINV,XMU,NCOLS,NCOLS,1,10,10,10,WORK)
      CALL MULTPY(XMUT,WORK,1,NCOLS,1,1,1,10,T1)
      TERM1=SE**2*T1(1,1)
      CALL MULTPY(VXX,B,NCOLS,NCOLS,1,10,10,10,WORK)
      CALL MULTPY(BT,WORK,1,NCOLS,1,1,1,10,T1)
      TERM2=T1(1,1)/DNY
      VMU=TERM1+TERM2
C
C     Compute COV
C
      CALL MULTPY(XTXINV,VXX,NCOLS,NCOLS,NCOLS,10,10,10,VS)
      CALL MULTPY(VS,B,NCOLS,NCOLS,1,10,10,10,VSB)
      CALL MULTPY(XMUT,VSB,1,NCOLS,1,1,1,10,T1)
      COV=2.0*SE**2*T1(1,1)
C
C     Compute VAR(var)
C
      DO 75 I=1,NCOLS
        DO 74 J=1,NCOLS
          DO 73 IH=1,NCOLS
            DO 72 IG=1,NCOLS
              A(I,J)=B(IH,1)*B(IG,1)*(VXX(I,IH)*VXX(IG,J)+
     1               VXX(I,J)*VXX(IH,IG))
 72         CONTINUE
 73       CONTINUE
          VXXT(I,J)=VXX(J,I)
 74     CONTINUE
 75   CONTINUE
      CALL MULTPY(VXXT,VSB,NCOLS,NCOLS,1,10,10,10,WORK)
      CALL MULTPY(BT,WORK,1,NCOLS,1,1,1,10,T1)
      TERM1=SE**4*(2.0/(NY-NCOLS)+4.0*T1(1,1))
      CALL MULTPY(A,B,NCOLS,NCOLS,1,10,10,10,WORK)
      CALL MULTPY(BT,WORK,1,NCOLS,1,1,1,10,T1)
      TERM2=T1(1,1)/(DNY-1.0)
      VARV=TERM1+TERM2
C
C     Write results
C
C     get station id of partial record site
      SAIND = 51
      SALEN = 1
      CALL WDBSGI (WDMSFL,DSN,SAIND,SALEN,ISTANO,RETFLG)
      IF (RETFLG .NE. 0) ISTANO = 0
      WRITE(CARD,2001) ISTANO
      WRITE(OFILE,2100) BLNK
      WRITE(OFILE,2100) BLNK
      WRITE(OFILE,2010) CARD
      YSD=DSQRT(YVAR)
      WRITE(CARD,2002)YMU(1,1)
      WRITE(OFILE,2100) BLNK
      WRITE(OFILE,2010) CARD
      WRITE(CARD,2003)         YSD
      WRITE(OFILE,2010) CARD
      WRITE(CARD,2004)             AVGSKU
      WRITE(OFILE,2010) CARD
C
C     put mean of logs (MEANND) on partial dataset
      SAIND = 280
      SALEN = 1
      TEMP = SNGL(YMU(1,1))
      CALL WDBSAR (WDMSFL,DSN,MESSFL,SAIND,SALEN,
     &             TEMP, RETCOD)
      IF (RETCOD .EQ. 0) THEN
C       Mean (&) added to data set.
        SGRP = 47
        CALL PRNTXR (MESSFL,SCLU,SGRP,TEMP)
      ELSE
C       Could not put mean on data set.
        SGRP = 48
        CALL PRNTXI (MESSFL,SCLU,SGRP,RETCOD)
      END IF
C     put stddev of logs (SDND) on partial record data set
      SAIND = 281
      TEMP = SNGL(YSD)
      CALL WDBSAR (WDMSFL,DSN,MESSFL,SAIND,SALEN,
     &             TEMP, RETCOD)
      IF (RETCOD .EQ. 0) THEN
C       Std dev (&) added to data set
        SGRP = 49
        CALL PRNTXR (MESSFL,SCLU,SGRP,TEMP)
      ELSE
C       Could not put stddev on data set.
        SGRP = 50
        CALL PRNTXI (MESSFL,SCLU,SGRP,RETCOD)
      END IF
C     put skew of logs (SKWND) on partial dataset
      SAIND = 282
      SALEN = 1
      CALL WDBSAR (WDMSFL,DSN,MESSFL,SAIND,SALEN,
     &             AVGSKU,RETCOD)
      IF (RETCOD .EQ. 0) THEN
C       Skew (&) added to data set.
        SGRP = 59
        CALL PRNTXR (MESSFL,SCLU,SGRP,AVGSKU)
      ELSE
C       Could not put skew on data set.
        SGRP = 60
        CALL PRNTXI (MESSFL,SCLU,SGRP,RETCOD)
      END IF
C
C     get TSTYPE
      SAIND = 1
      SALEN = 4
      CALL WDBSGC (WDMSFL,DSN,SAIND,SALEN,CTYPE,
     &             RETCOD)
C
C     set window name for next screen
      WNDNAM = 'Put (PASP)'
      CALL ZWNSET (WNDNAM)
      WRITE(CARD,2005)
      WRITE(OFILE,2100) BLNK
      WRITE(OFILE,2100) BLNK
      WRITE(OFILE,2010) CARD
      CALL CVARAR (L78,CARD,L78,BUFF)
      INIT = 1
      IWRT = 1
      CALL ZBLDWR (L78,BUFF,INIT,IWRT,DONEFG)
      INIT = 0
      DO 85 I=1,4
C       loop for each return period (2- 5- 10- 20-)
        P=1.0/R(I)
        AK(I)=WILFRT(AVGSKU,GAUSAB(P),ERRFLG)
        YHAT(I)=YMU(1,1)+DBLE(AK(I))*DSQRT(YVAR)
C
        CALL CHRCHR (L2,CTYPE(3),ATTR(2,I))
        CALL WDBSGX (MESSFL, 
     M               ATTR(1,I), 
     O               SAIND, SATYP, SALEN)
C       store as attributes
        TEMP = 10.0**(SNGL(YHAT(I)))
        CALL WDBSAR (WDMSFL,DSN,MESSFL,SAIND,SALEN,
     &                 TEMP,ERRCOD(I))
        IF (ERRCOD(I) .EQ. 0) THEN
C
          VYHAT(I)=VMU+DBLE(AK(I))**2/(4.0*YVAR)*VARV+DBLE(AK(I))/
     &             DSQRT(YVAR)*COV
          HARD=1.0+.5*DBLE(AK(I)**2)
          EQYRS(I)=HARD*YVAR/VYHAT(I)
          IF (EQYRS(I) .GT. DBLE(NY)) EQYRS(I) = DBLE(NY)
          IF (EQYRS(I) .LT. DBLE(3)) EQYRS(I) = DBLE(3)
          WRITE(CARD,2000)R(I),TEMP,YHAT(I),VYHAT(I),EQYRS(I)
          WRITE(OFILE,2010) CARD
          CALL CVARAR (L78,CARD,L78,BUFF)
C         wait for user acknowledgement after last return period
          IF (I .EQ. 4) IWRT = 0
          CALL ZBLDWR (L78,BUFF,INIT,IWRT,DONEFG)
        END IF
 85   CONTINUE
C
      DO 95 I = 1,4
        IF (ERRCOD(I) .EQ. 0) THEN
C         N-yr low-flow put on data set.
          OTYP(1) = 2
          OTYP(2) = 4
          TEMP = 10.0**(SNGL(YHAT(I)))
          IWRT = 0
          SGRP = 50 + I
          CALL PMXTXM (MESSFL,SCLU,SGRP,ML,SF,L2,OTYP,
     &                 IWRT,IDUM,TEMP,DVAL,L6,ATTR(1,I))
        ELSE
C         Could not put n-yr event on data set. Error is &.
          SGRP = 54 + I
          CALL PRNTXI (MESSFL,SCLU,SGRP,ERRCOD(I))
        END IF
 95   CONTINUE
C
C     add number of years of low-flow record (YRSLOW) to dataset 
      SAIND = 219
      SALEN = 1
      TEMP = SNGL(EQYRS(1))
      CALL WDBSAR (WDMSFL,DSN,MESSFL,SAIND,SALEN,
     &             TEMP, RETCOD)
      IF (RETCOD .EQ. 0) THEN
C       Equivalent years of record (&) added to dataset
        SGRP = 61
        CALL PRNTXR (MESSFL,SCLU,SGRP,TEMP)
      ELSE
C       Could not put equiv yrs rec on data set.
C       error is &.
        SGRP = 62
        CALL PRNTXI (MESSFL,SCLU,SGRP,RETCOD)
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   PSTYRS
     I                   (MESSFL, SCLU, WDMSFL, NSTAT, DSNP, NPTS, IYR)
C
C     + + + PURPOSE + + +
C     Puts an annual time series on a set of data sets, putting a value\
C     of 1.0 for each year specified in the IYR array.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER      MESSFL, SCLU, WDMSFL, NSTAT, DSNP(NSTAT), NPTS,
     $             IYR(100)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MESSFL - Fortran unit number of message file
C     SCLU   - screen cluster number on message file
C     WDMSFL - Fortran unit number of wdm file
C     NSTAT  - number of annual series for partial record sta. in buffer
C     DSNP   - array containing up to 12 data sets of annual low-flow 
C              series to add partial record estimates
C     NPTS   -
C     IYR    - calendar year of discharge measurement
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      I, K, L100, N, NY, TS, TU, DTOVWR, SDATE(6), QFLAG,
     $             SGRP, INUM, IVAL(2), MAXLIN, SCNFG, IWRT, RETCOD
      REAL         RNUM(100), RDEF
C
C     + + + EXTERNALS + + +
      EXTERNAL     WDTPUT
      EXTERNAL     PMXTXI
      EXTERNAL     ASRTIP,ZIPR 
C
C     + + + DATA INITIALIZATIONS + + +
      DATA SDATE /0,1,1,0,0,0/
C
C     + + + END SPECIFICATIONS + + +
C
      TS = 1
      TU = 6
      L100 = 100
      RDEF = -999999.
      DTOVWR = 1
      QFLAG = 0
      MAXLIN = 10
      SCNFG = 1
C
C     sort years
      CALL ASRTIP (NPTS,IYR)
      CALL ZIPR (L100,RDEF,RNUM)
      NY = IYR(NPTS) - IYR(1) + 1
      SDATE(1) = IYR(1)
      DO 10 I = 1, NPTS
        K = IYR(I) - IYR(1) + 1
        RNUM(K) = 1.0
 10   CONTINUE
      DO 20 N = 1, NSTAT
        CALL WDTPUT (WDMSFL, DSNP(N), TS, SDATE, NY, DTOVWR,  
     I               QFLAG, TU, RNUM, 
     O               RETCOD)
        IF (RETCOD .NE. 0) THEN
C         Time series was not added for data set &. Error code &.
          IWRT = 0
          INUM = 2
          IVAL(1) = DSNP(N)
          IVAL(2) = RETCOD
          SGRP = 90
          CALL PMXTXI (MESSFL, SCLU, SGRP, MAXLIN, SCNFG, IWRT, INUM, 
     I                 IVAL)
        ELSE
C         1.0 added for & years for time-series data set &
          IWRT = 0
          INUM = 2
          IVAL(1) = NY
          IVAL(2) = DSNP(N)
          SGRP = 95
          CALL PMXTXI (MESSFL, SCLU, SGRP, MAXLIN, SCNFG, IWRT, INUM, 
     I                 IVAL)
        END IF
 20   CONTINUE
C
      RETURN
      END
