#!/bin/sh
# test.sh -- run glsnet test data sets
#
# Usage: test.sh [start [stop [size] ] ]
#
#        where: start = starting test number
#                stop = ending test number (may be same as start)
#                size = max no of stations glsnet dimensioned for
#                       valid values are 300 (default), 600, and 700
#
# History: 95/12/08 kmflynn
#          97/06/11 kmflynn  add selection of different dimensions

# pathname and version number
  WrdA=/usr/opt/wrdapp
  Vers=2.5

# version of gks being used, set Gks to
#        Xgks - if using Xgks library for graphics
#        PriorGks - if using Prior Gks library for graphics
    Xgks=x
PriorGks=
     Gks=$PriorGks

# glsnet may be provided in three different sizes,
# dimensioned for up to 300, 600, or 700 stations.
# default size is 300
  Size=${3-300}

#*******************************************************************
#***** You should not need to modify anything below this line. *****
#*******************************************************************

# pathnames and other variables
  Prgm=$WrdA/glsnet$Vers
  Data=$Prgm/data
  Chck=$Prgm/test

#
  exec 2>&1                                # stderr shows up in .out file
  Start=${1-1}                             # by default, start at 1
  Stop=${2-2}                              # by default, stop at 2
                                           # 1 - texas peaks
                                           # 2 - virginia low flows
#
# begin test runs
  echo
  echo _________________________________________________________________
  echo "Begin processing glsnet test runs $Start to $Stop"
  echo " glsnet program from: $Prgm"
  echo "      dimensioned to: $Size"
  echo "      test data from: $Data"
  echo `date`

  Test=$Start

  while [ $Test -ge $Start -a $Test -le $Stop - $Test -le 2 ] ; do
#   glsnet test runs
    echo
    echo
    echo _________________________________________________________________
    echo "Test run number $Test"
    echo
#   remove old output files
    for File in out ot1 ot2 ; do
      if [ -f Test$Test.$File ] ; then rm Test$Test.$File ; fi
    done

#   copy (modified) or link (unchanged) data files
    if [ $Test -eq 1 ] ; then ln -s $Data/tx.wdm tx.wdm ; fi
    if [ $Test -eq 2 ] 
      then
      cp -p $Data/va.wdm va.wdm
      ln -s $Data/pr_01632 pr_01632
      ln -s $Data/pr_02018 pr_02018
    fi

    $Prgm/bin/gls$Size.exe <<-EOT
	@$Data/test$Test.log
	EOT

    Test=`expr $Test + 1`
  done

# remove unneeded files
  for File in tx.wdm va.wdm pr_01632 pr_02018 ; do
    if [ -f $File ] ; then rm $File ; fi
  done
#
  echo
  echo
  echo _________________________________________________________________
  echo "Completed glsnet test runs $Start to $Stop"
  echo
# check output against original output in $Data directory
  $Chck/check.sh $Data

# end of shell
