#!/bin/sh
# wdimex.sh -- make the message file and test wdm files for glsnet
#
# History:  96/02/16 kmflynn
#           00/11/07 kmflynn
#
# Usage: wdimex.sh [ file_name ] [| tee output_file ]
#
# Where: file_name = the name of the message file or test wdm file to build
#                    glsmes - to generate the glsnet message file (default)
#                        tx - generate texas peak flows wdm file
#                        va - generate virginia low flows wdm file
#
# Examples:  wdimex.sh
#            wdimex.sh va

# pathnames and directories
  WrdA=/usr/opt/wrdapp
  Libr=$WrdA/libanne4.1

#*******************************************************************
#***** You should not need to modify anything below this line. *****
#*******************************************************************

Name=${1-'glsmes'}
     if [ ! $Name ] ; then Name=glsmes ; fi

# files for message file
  SeqMsg='glsnet algmes erranl netwrk progls prosls prtpro prtste'
  SeqLib='ann/qtdate    waide/agplot  waide/tbltmp'

  echo
  echo ' ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ' ; echo
  echo '     building wdm file(s) for:' $Name                       ; echo
  echo '              Library =' $Libr
  echo

  if [ $Name = 'glsmes' ] ; then
#    build the message file
     echo ' ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ' ; echo
     echo '             wdm name =' $Name.wdm
     echo '          sequentials =' $SeqMsg
                     for Seq in $SeqLib ; do
     echo '                       ' $Seq
                     done
     echo
     echo ' ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ' ; echo

#    remove any old files
     if [ -f error.fil ] ; then rm error.fil ; fi
     if [ -f $Name.wdm ] ; then rm $Name.wdm ; fi
     if [ -f $Name.in  ] ; then rm $Name.in  ; fi
     if [ -f $Name.err ] ; then rm $Name.err ; fi

#    build input file
     echo $Name.wdm > $Name.in
     for Seq in $SeqMsg ; do
        echo I >> $Name.in
        echo $Seq.seq >> $Name.in
     done
     for Seq in $SeqLib ; do
        echo I >> $Name.in
        echo $Libr/msg/$Seq.seq >> $Name.in
     done
     echo R >> $Name.in

#    get message file shell
     cp $Libr/lib_data/message.wdm $Name.wdm

#    build message file
     $Libr/bin/wdimex < $Name.in

#    remove old message file and move new one
     if [ -f ../bin_data/$Name.wdm ] ; then rm ../bin_data/$Name.wdm ; fi
     mv $Name.wdm ../bin_data
     mv error.fil $Name.err

  else
#    build a test wdm file


     echo ' ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ' ; echo
     echo '             wdm name =' $Name.wdm
     echo '            data file =' $Name.exp
     echo
     echo ' ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ' ; echo

#    remove any old files
     if [ -f error.fil ] ; then rm error.fil ; fi
     if [ -f $Name.wdm ] ; then rm $Name.wdm ; fi
     if [ -f $Name.in  ] ; then rm $Name.in  ; fi
     if [ -f $Name.err ] ; then rm $Name.err ; fi


#    build input file
     echo $Name.wdm > $Name.in
     echo c >> $Name.in
     echo n >> $Name.in
     echo ../bin_data/glsmes.wdm >> $Name.in
     echo i >> $Name.in
     echo ../data/$Name.exp >> $Name.in
     echo r >> $Name.in

#    build the test wdm file
     $Libr/bin/wdimex < $Name.in

#    remove old test wdm file and move new one into data
     if [ -f ../data/$Name.wdm ] ; then rm ../data/$Name.wdm ; fi
     mv $Name.wdm ../data/$Name.wdm
     mv error.fil $Name.err
  fi

# end of shell
