C
C
C
      SUBROUTINE   PRTSTE
     I                   ( MESSFL, WDMSFL, ERRFL, NSTAT, DSNP, NDAY )
C
C     + + + PURPOSE + + +
C     This routine adds data for a partial record site using the
C     Stedinger technique
C
C     + + + HISTORY + + +
C     kmf - May 19, 2004 - Combined Output and Input options; the
C                          output file is now specified in pstidx on
C                          the same form as the input file.  Close
C                          the output file before leaving prtste.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     MESSFL, WDMSFL, ERRFL, NSTAT,
     $            DSNP(NSTAT), NDAY(NSTAT)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MESSFL - Fortran unit number of message file
C     WDMSFL - Fortran unit number of WDM file
C     ERRFL  - Fortran unit number for diagnostic output
C     NSTAT  - Number of annual series for partial record
C              station in the buffer
C     DSNP   - Datasets of station for up to 12 annual low flow
C              series to add partial record estimates.
C     NDAY   - low flow durations for each data set
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      SCLU, SGRP, RESP, OFILE, ANAFG, RETCOD,
     $             FLAG, NIDX, NOBS, KEEP,
     $             IDXSTA(6), DSNI(6,12),IYR(100)
      REAL         PRED(100), COOKD(100), RESID(100), YPR(100),
     $             XIS(100,6)
      DOUBLE PRECISION   XTXINV(10,10), B(10,1), SE
C
C     + + + LOCAL DEFINITIONS + + +
C     ANAFG  - indicator flag for analysis
C              0 - data has not been analyzed
C              1 - data has been analyzed
C
C     + + + EXTERNALS + + +
      EXTERNAL     QRESP, PRNTXT
      EXTERNAL     PSTIDX, PSTPLT, PSTANL, PSTCOM
C
C     + + + END SPECIFICATIONS + + +
C
      SCLU = 153
      OFILE= 0
      ANAFG= 0
      RESP = 1
 10   CONTINUE
C       Stedinger data add option:
C            1-Identify, 2-Analyze, 3-Plot, 4-Write, 5-Return
        SGRP= 1
        CALL QRESP ( MESSFL, SCLU, SGRP, RESP )
        IF (RESP .EQ. 1) THEN
C         Identify file names and index stations
          CALL PSTIDX ( MESSFL, SCLU, WDMSFL, ERRFL, NSTAT, NDAY,
     M                  OFILE,
     O                  NOBS, YPR, XIS, IYR, NIDX, DSNI, IDXSTA,
     O                  RETCOD )
          IF (RETCOD .EQ. 0) THEN
C           data ok, continue
            RESP = 2
          ELSE
C           problem
            RESP = 5
          END IF
        ELSE IF (RESP .EQ. 2) THEN
C         Analyze data at index sites
          IF (NOBS .GT. 0  .AND.  OFILE .GT. 0) THEN
C           files and data ok, analyze
            CALL PSTANL (MESSFL,SCLU,OFILE,NOBS,NIDX,IDXSTA,
     M                   YPR,XIS,
     O                   B,SE,XTXINV,RESID,COOKD,PRED,FLAG)
            IF (FLAG .EQ. 0) ANAFG = 1
            RESP = 3
          ELSE
C           need to Identify output file and index sites
            SGRP = 7
            CALL PRNTXT ( MESSFL, SCLU, SGRP )
            RESP = 1
          END IF
        ELSE IF (RESP .EQ. 3) THEN
C         Plot results of analysis
          IF (ANAFG .EQ. 1) THEN
C           do plots
            CALL PSTPLT (MESSFL,SCLU,NOBS,YPR,PRED,RESID,COOKD)
            RESP = 4
          ELSE
C           must do analysis first
            SGRP = 9
            CALL PRNTXT ( MESSFL, SCLU, SGRP )
            RESP = 2
          END IF
        ELSE IF (RESP .EQ. 4) THEN
C         compute statistics and Write attributes and years
          IF (ANAFG .EQ. 1) THEN
            CALL PSTCOM ( MESSFL, SCLU, WDMSFL, OFILE,
     I                    NSTAT, NIDX, DSNI, DSNP,
     I                    B, SE, XTXINV )
            CALL PSTYRS ( MESSFL, SCLU, WDMSFL,
     I                    NSTAT, DSNP, NOBS, IYR )
            RESP = 5
          ELSE
C           must do analysis first
            SGRP = 12
            CALL PRNTXT ( MESSFL, SCLU, SGRP )
            RESP = 2
          END IF
        END IF
      IF (RESP .NE. 5) GO TO 10
C
      IF (OFILE .NE. 0) THEN
C       close the output file
        KEEP = 0
        CALL QFCLOS ( OFILE, KEEP )
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   PSTIDX
     I                   ( MESSFL, SCLU, WDMSFL, ERRFL, NSTAT, NDAY,
     M                     OFILE, 
     O                     NOBS, YPR, XIS, IYR, NIDX, DSNI, IDXSTA,
     O                     RETC )
C
C     + + + PURPOSE + + +
C     This routine opens the output file and the sequential input
C     file of partial record and index sites, reads the input file
C     and checks matches on station numbers.
C
C     + + + HISTORY + + +
C     kmf - May 19, 2004 - replaced calls to prntxt and prntxi
C                          with writes to prtrow which is passed
C                          to q2edit.  Modified existing formats
C                          and added new ones.  Moved open of output
C                          file into pstidx from xxxxx and combined
C                          with opening input file - done with call
C                          to ropnio.  Replaced listing of possible
C                          index stations and asking user to enter
C                          desired index station numbers with a
C                          parm2 screen asking users to indicate
C                          yes or no to include possible index
C                          stations.  If an output file is already
C                          open, it can be used.
C     kmf - Sep 24, 2004 - changed valid time period from 1800-2010
C                          to parameters yrfrst-yrlast (1800-2049).
C                          Added parameter include file pmxyrs.inc.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   MESSFL, SCLU, WDMSFL, ERRFL, NSTAT, NDAY(NSTAT),
     $          NOBS, IYR(100), NIDX, DSNI(6,12), IDXSTA(6),
     $          OFILE, RETC
      REAL      YPR(100), XIS(100,6)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MESSFL - Fortran unit number of message file
C     SCLU   - cluster number on message file
C     WDMSFL - Fortran unit number of WDM file
C     ERRFL  - Fortran unit number for reporting problems
C     NSTAT  - number of annual series for the station
C     NDAY   - low flow durations for each data set
C     NOBS   - number of concurrent discharges at partial and index
C              stations
C     YPR    - discharge at partial record station
C     XIS    - discharge at index sites
C     IYR    - calendar year of discharge measurement
C     NIDX   - number of index sites
C     DSNI   - data set numbers at index sites
C     IDXSTA - station numbers of index sites
C     OFILE  - Fortran unit number of output file
C     RETC   - return code
C
C
C     + + + PARAMETERS + + +
      INCLUDE 'pmxyrs.inc'
C
C     + + + LOCAL PARAMETERS + + +
      INTEGER       TNIDX
      PARAMETER    (TNIDX = 10)
C
C     + + + LOCAL VARIABLES + + +
      INTEGER       INUNIT, RDERR, KNT, IPT, I, J, K, L1, L78,
     $              IVAL(TNIDX), ICOL(TNIDX), INCL(TNIDX), ORDER(TNIDX),
     $              CONT, AGAIN, MORE, FLAG, NPRT, YEAR, ITEMP,
     $              SGRP, SGRP2, SGRPQ, RESP, IRET, KEEP, MAR, APR, MO
      REAL          PX(TNIDX)
      CHARACTER*4   TSTYPE(12)
      CHARACTER*78  PRTROW(5)
      CHARACTER*112 STR
C
C     + + + LOCAL DEFINITIONS + + +
C     TNIDX  - maximum number of index sites in the input file
C     ORDER  - field(s) containing selected index station(s)
C              max of 6 may be selected, but this is dimenstioned
C              to TNIDX to avoid subscript error if user selects
C              more than 6 index stations.  User will be notified
C              and corrective action will be taken.
C
C     + + + FUNCTIONS + + +
      INTEGER   YRSEAS
C
C     + + + EXTERNALS + + +
      EXTERNAL  Q2INIT, Q2SETI, Q2STCO, Q2SCTF, Q2EDIT, Q2GTCO, QRESP
      EXTERNAL  QFCLOS
      EXTERNAL  GTDSNS, YRSEAS
C
C     + + + INPUT FORMATS + + +
 1002 FORMAT ( A112 )
 1005 FORMAT ( I9 )
 1305 FORMAT ( I4, I2 )
 1310 FORMAT ( 11F9.0 )
C
C     + + + OUTPUT FORMATS  + + +
 2010 FORMAT (5X, '*** No output file opened.' )
 2011 FORMAT (5X, '*** No input file opened.' )
 2015 FORMAT (    '*** ',
     $         /, '*** No output file opened.',
     $         /, '*** ' )
 2016 FORMAT (    '*** ',
     $         /, '*** No input file opened.',
     $         /, '*** ' )
 2102 FORMAT (    '*** ',
     $         /, '*** Read error on first record in input file',
     $         /, '***',
     $         /, '***      iostat = ', I5,
     $         /, '*** ' )
 2110 FORMAT (    '*** ',
     $         /, '*** Problem with first record in input file',
     $         /, '*** The first record is expected to contain ',
     $                 '(in the indicated columns):',
     $         /, '***          1 - "S" ',
     $         /, '***      14-22 - partial record station number ',
     $                 '(optional)',
     $         /, '***      23-31 - index station (required)',
     $         /, '***      32-40 - index station (optional)',
     $         /, '***       ...  - index stations, 9 spaces each',
     $         /, '***     104-112- index station (optional)',
     $         /, '*** There must be at least 1 index station.',
     $         /, '***',
     $         /, '*** The first record looks like:',
     $         /, '*** ', A70, ' ...',
     $         /, '*** S            <-------><-------><-------> ...',
     $         /, '*** ----+----1----+----2----+----3----+----4 ...',
     $         /, '*** ' )
 2300 FORMAT (    '*** ',
     $         /, '*** Error reading discharges at or near ',
     $                 'record', I5,
     $         /, '*** The records are expected to contain ',
     $                 '(in the indicated columns):',
     $         /, '***       2-5  - year (', I4, ' - ', I4, ')',
     $         /, '***       6-7  - month',
     $         /, '***       8-9  - day',
     $         /, '***      11-13 - counter',
     $         /, '***      14-22 - partial record Q',
     $         /, '***      23-31 - index station Q',
     $         /, '***      32-40 - index station Q',
     $         /, '***       ...  - index station Q',
     $         /, '***     104-112- index station Q',
     $         /, '***',
     $         /, '*** The record looks like:',
     $         /, '*** ', A,
     $         /, '***  yyyymmdd nnn<-------><-------><-------> ...',
     $         /, '*** ----+----1----+----2----+----3----+----4 ...',
     $         /, '*** ' )
 2305 FORMAT (    '*** At or near record', I4,
     $            ' the year field contained "', A4, '"' )
 2306 FORMAT (    '*** At or near record', I4,
     $            ' found invalid year ', I4 )
 2307 FORMAT (    '*** Problem reading discharges at or near record',
     $            I4, ', year =', I5 )
 2308 FORMAT (    'Too many records in input file, maximum allowed ',
     $            'is 100 years.' )
 2309 FORMAT (    'Not enough years, at least 4 required' )
 2310 FORMAT (5X, 'Ready to process:', I4, ' concurrent discharges ',
     $            'read for', I2, ' index station(s).' )
C
C     + + + DATA INITIALIZATION + + +
      DATA    L1, L78, KEEP, MAR, APR
     $      /  1,  78,    0,   3,   4 /
C
C     + + + END SPECIFICATIONS + + +
C
      NOBS  = 0
      RETC  = 0
      CONT  = 1
      NPRT  = 0
      SGRPQ = 0
C
C     get output and input file names
      SGRP  = 2
      SGRP2 = 4
      CALL ROPNIO ( MESSFL, SCLU, SGRP, SGRP2, OFILE, INUNIT )
      IF (OFILE .EQ. 0) THEN
C       no output file opened
        SGRPQ = 64
        CONT  = 0
        NPRT  = NPRT + 1
        WRITE (PRTROW(NPRT),2010)
        WRITE (ERRFL,2015)
      END IF
      IF (INUNIT .EQ. 0) THEN
C       no input file opened
        SGRPQ = 64
        CONT  = 0
        NPRT  = NPRT + 1
        WRITE (PRTROW(NPRT),2011)
        WRITE (ERRFL,2016)
      END IF
C
      IF (CONT .EQ. 1) THEN
C       successfully opened files, read first record
        READ (INUNIT,1002,IOSTAT=RDERR) STR
        IF (RDERR .NE. 0) THEN
C         read error
          SGRPQ = 71
          CONT  = 0
          WRITE (ERRFL,2102) RDERR
        ELSE IF (STR(1:1) .NE. 'S') THEN
C         first record must start with STATION
          SGRPQ = 71
          CONT  = 0
          WRITE (ERRFL,2110) STR(1:70)
        END IF
      END IF
C
      IF (CONT .EQ. 1) THEN
C       first record looks ok, get index stations
        KNT = 0
        IPT = 23
        DO 100 I = 1, TNIDX
          READ (STR(IPT:IPT+8),1005,IOSTAT=RDERR) ITEMP
          IF (RDERR .NE. 0  .AND.  CONT .EQ. 1) THEN
C           read error
            SGRPQ = 71
            CONT  = 0
            WRITE (ERRFL,2110) STR(1:70)
          ELSE IF (ITEMP .GT. 0) THEN
C           found an index station, save it
            KNT = KNT + 1
            IVAL(KNT) = ITEMP
            ICOL(KNT) = I
            INCL(KNT) = 2
          END IF
          IPT = IPT + 9
 100    CONTINUE
        IF (KNT .EQ. 0  .AND.  CONT .EQ. 1) THEN
C         no index stations found, need at least 1
          SGRPQ = 71
          CONT  = 0
          WRITE (ERRFL,2110) STR(1:70)
        END IF
      END IF
C
      IF (CONT .EQ. 1) THEN
C       input file contains index stations
 200    CONTINUE
C         which index stations should be used?
          SGRP = 5
 210      CONTINUE
            AGAIN = 0
            CALL Q2INIT ( MESSFL, SCLU, SGRP )
            CALL Q2SETI ( L1, KNT, IVAL )
            CALL Q2STCO ( L1, KNT, INCL )
            CALL Q2EDIT ( KNT, IRET )
            IF (IRET .EQ. -1) THEN
C             oops, try again
              AGAIN = 1
            ELSE IF (IRET .EQ. 2) THEN
C             previous, take no action
              AGAIN = -1
              CONT  = 0
            ELSE
C             assume user wants to continue (1)
              CALL Q2GTCO ( L1, KNT, INCL )
      write (99,*) ' <-- q2gtco:  l1 =', l1
      write (99,*) '             knt =', knt
      write (99,*) '            incl =', (incl(i),i=1,knt)
              NIDX = 0
              DO 250 I = 1, KNT
                IF (INCL(I) .EQ. 1) THEN
C                 include this station
                  NIDX = NIDX + 1
                  IDXSTA(NIDX) = IVAL(I)
                  ORDER(NIDX)  = ICOL(I)
                END IF
 250          CONTINUE
              iF (NIDX .LT. 1  .OR.  NIDX .GT. 6) THEN
C               select at least 1 at most 6, use Prev (F4) to give up
                SGRP  = 10
                AGAIN = 1
              END IF
            END IF
          IF (AGAIN .EQ. 1) GO TO 210
          IF (CONT .EQ. 1) THEN
C           index stations identified, find dsn on wdm file
            CALL GTDSNS ( WDMSFL, MESSFL, SCLU, NSTAT,
     I                     NIDX, IDXSTA, NDAY,
     O                     TSTYPE, DSNI, FLAG )
            IF (FLAG .GT. 0) THEN
C             did not find all index stations on the wdm file,
C             1-exit glsnet
C             1-Select different index station(s)
C             2-Return to Stedinger menu
              SGRP = 82
              CALL QRESP ( MESSFL, SCLU, SGRP, RESP )
              IF (RESP .EQ. 1) THEN
C               Return to Stedinger menu
                CONT  = 0
                AGAIN = -1
              ELSE
C               select different index station(s)
                SGRP  = 10
                AGAIN = 1
              END IF
            END IF
          END IF
        IF (CONT .EQ. 1  .AND.  AGAIN .EQ. 1) GO TO 200
      END IF
C
      IF (CONT .EQ. 1) THEN
C       valid index stations identified, read data
        I = 0
        MORE   = 1
 300    CONTINUE
          READ (INUNIT,1002,END=390) STR
C           got a record, process it
Ckmf---------------------------------------------------------Note
Ckmf                                                         Note
Ckmf        code does not account for difference between     Note
Ckmf        calender year and climatic year.  Revised to     Note
Ckmf        adjust year as appropriate based on month.       Note
Ckmf        if month is missing from the input file, the     Note
Ckmf        year is assumed to be correct.                   Note
Ckmf                                                         Note
Ckmf        revised code to consider this                    Note
Ckmf                                                         Note
Ckmf        READ (STR(2:5),1305,IOSTAT=RDERR) YEAR
Ckmf                                                         Note
Ckmf---------------------------------------------------------Note
            READ (STR(2:7),1305,IOSTAT=RDERR) YEAR, MO
            IF (RDERR .NE. 0) THEN
C             read error
              SGRPQ = 36
              CONT  = 0
              NPRT  = NPRT + 1
              WRITE (PRTROW(NPRT),2305) I+1, STR(2:5)
              WRITE (ERRFL,2300) I+1, YRFRST, YRLAST, STR
            ELSE IF (YEAR .LT. YRFRST  .OR.  YEAR .GT. YRLAST) THEN
C             invalid year
              SGRPQ = 36
              CONT  = 0
              NPRT  = NPRT + 1
              WRITE (PRTROW(NPRT),2306) I+1, YEAR
              WRITE (ERRFL,2300) I+1, YRFRST, YRLAST, STR
            ELSE IF (I .LT. 100) THEN
C             year ok and still room, read Q's
              I = I + 1
              READ (STR(14:111),1310,IOSTAT=RDERR)
     $             YPR(I), (PX(J), J = 1, TNIDX)
              IF (RDERR .NE. 0) THEN
C               read error
                SGRPQ = 36
                CONT  = 0
                NPRT  = NPRT + 1
                WRITE (PRTROW(NPRT),2307) I, YEAR
                WRITE (ERRFL,2300) I, YRFRST, YRLAST, STR
              ELSE
                DO 350 J = 1, NIDX
                  K = ORDER(J)
                  XIS(I,J) = PX(K)
 350            CONTINUE
Ckmf---------------------------------------------------------Note
Ckmf                                                         Note
Ckmf            code does not account for difference between Note
Ckmf            calender year and climatic year.  Revised to Note
Ckmf            adjust year as appropriate based on month.   Note
Ckmf            if month is missing from the input file, the Note
Ckmf            year is assumed to be correct.               Note
Ckmf                                                         Note
Ckmf            revised code to consider this                Note
Ckmf                                                         Note
Ckmf            IYR(I) = YEAR
Ckmf                                                         Note
Ckmf---------------------------------------------------------Note
                IYR(I) = YRSEAS( APR, MAR, YEAR, MO )
              END IF
            ELSE
C             maximum of 100 Q's possible
              SGRPQ = 37
              CONT  = 0
              MORE  = 0
            END IF
            GO TO 399
 390      CONTINUE
C           end of file
            MORE = 0
            IF (I .GE. 4) THEN
C             enough data
              SGRPQ = 38
              NOBS  = I
              NPRT  = NPRT + 1
              WRITE (PRTROW(NPRT),2310) NOBS, NIDX
            ELSE
C             not enough data, need at least 4
              SGRPQ = 39
              CONT  = 0
              NOBS  = 0
            END IF
 399      CONTINUE
        IF (MORE .EQ. 1  .AND.  CONT .EQ. 1) GO TO 300
      END IF
C
      IF (INUNIT .NE. 0) THEN
C       close input file and free up unit number
        CALL QFCLOS ( INUNIT, KEEP )
      END IF
C
      IF (SGRPQ .NE. 0) THEN
C       send message to screen
        IF (NPRT .EQ. 0) THEN
C         simple message
          CALL PRNTXT ( MESSFL, SCLU, SGRPQ )
        ELSE
C         details included
          CALL Q2INIT ( MESSFL, SCLU, SGRPQ )
          CALL Q2SCTF ( L1, L78, NPRT, PRTROW )
          CALL Q2EDIT ( NPRT, IRET )
        END IF
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   GTDSNS
     I                   (WDMSFL, MESSFL, SCLU, NSTAT,
     I                    NIDX, IDXSTA, NDAY,
     O                    TSTYPE, DSNI, RETCOD)
C
C     + + + PURPOSE + + +
C     This routine finds time series data set numbers given station
C     id and TSTYPE for n-day annual series.
C
C     + + + HISTORY + + +
C     kmf - May  6, 2004 - replaced calls to prntxi and pmxtxi
C                          with writes to prtrow which is passed
C                          to q2edit.  Modified existing formats
C                          and added new ones.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER      WDMSFL, MESSFL, SCLU, NSTAT, NIDX, DSNI(6,12),
     $             RETCOD, IDXSTA(NIDX), NDAY(NSTAT)
      CHARACTER*4  TSTYPE(12)
C
C     + + + ARGUMENT DEFINITION + + +
C     WDMSFL - Fortran unit number of wdm file
C     MESSFL - Fortran unit number of message file
C     SCLU   - message file group
C     NSTAT  - number of n-day types
C     NIDX   - number of index stations
C     IDXSTA - array of index stations to be found
C     NDAY   - array of n-day time series for index sta.
C     TSTYPE - array of tstype attributes looked for
C     DSNI   - array of data set numbers found for index sta.
C     RETCOD - return code
C              0 - no problem
C              >0 - number of data sets not identified
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      DTYPE, TU, TS, DSMAX, I, J, N, L1, L78,
     $             SGRP, IRET, DSNKNT
      CHARACTER*1  CSTA(16)
      CHARACTER*78 PRTROW(72)
C
C     + + + EXTERNALS + + +
      EXTERNAL   INDSFN, Q2INIT, Q2SCTF, Q2EDIT
C
C     + + + FORMATS + + +
 2010 FORMAT ( 'L', I3.3 )
 2040 FORMAT ( I12, 5X, A4,     9X, 'Problem finding data set',
     $                              '  !! WARNING !!' )
 2045 FORMAT ( I12, 5X, A4, I7, 2X, 'Successfully found dsn ' )
C
C     + + + DATA INITIALIZATION + + +
      DATA  DSMAX, DTYPE, TS, TU, CSTA,   L1, L78
     $     /    1,     1,  1,  6, 16*' ',  1,  78 /
C
C     + + + END SPECIFICATIONS + + +
C
      RETCOD = 0
C     determine TSTYPE
      DO 10 N = 1,NSTAT
        WRITE (TSTYPE(N),2010) NDAY(N)
 10   CONTINUE
C
C     find data set number(s)
      J = 0
      DO 50 I = 1,NIDX
C       for index stations for each n-day statistic
        DO 40 N = 1,NSTAT
C         for each n-day statistic
          CALL INDSFN (WDMSFL, DTYPE, TSTYPE(N), CSTA, IDXSTA(I),
     I                 TS, TU, DSMAX,
     O                 DSNI(I,N), DSNKNT)
          J = J + 1
          IF (DSNKNT .EQ. 0) THEN
C           data set for index station not found
            WRITE (PRTROW(J),2040) IDXSTA(I), TSTYPE(N)
            RETCOD = RETCOD + 1
          ELSE
C           Data set & found for station &.
            WRITE (PRTROW(J),2045) IDXSTA(I), TSTYPE(N), DSNI(I,N)
          END IF
 40     CONTINUE
 50   CONTINUE
C
C     report data sets found and not found to screen
      SGRP = 84
      CALL Q2INIT ( MESSFL, SCLU, SGRP )
      CALL Q2SCTF ( L1, L78, J, PRTROW )
      CALL Q2EDIT ( J, IRET )
C
      RETURN
      END
C
C
C
      SUBROUTINE   PSTANL
     I                  (MESSFL,SCLU,OFILE,NOBS,NIDX,IDXSTA,
     M                   YPR,XIS,
     O                   B,SE,XTXINV,RESID,COOKD,PRED,FLAG)
C
C     + + + PURPOSE + + +
C     This routine performs ordinary least squares on data necessary for
C     the Stedinger-Thomas method to estimate low flow statistics for
C     the partial record site.
C
C     + + + HISTORY + + +
C     kmf - Apr 2, 2002 - added message file unit and cluster to call
C                         to LREG6
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   MESSFL,SCLU,OFILE,NOBS,NIDX,IDXSTA(NIDX),FLAG
      REAL   PRED(NOBS),RESID(NOBS),COOKD(NOBS),YPR(100),XIS(100,6)
      DOUBLE PRECISION   B(10,1),SE,XTXINV(10,10)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MESSFL - Fortran unit number of message file
C     SCLU   - message file group
C     OFILE  - Fortran unit number of output file
C     NOBS   - number of concurrent discharges at partial and index
C              stations
C     NIDX   - number of index sites
C     IDXSTA - station numbers of index sites
C     YPR    - discharges at partial record station
C     XIS    - discharges at index sites
C     B      - regression coefficients
C     SE     - standard error of estimate
C     XTXINV - inverse of XTX matrix
C     RESID  - residual
C     COOKD  - Cook's D statistic
C     PRED   - predicted values
C     RETCOD - 1 if too few points for analysis, else 0
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   SGRP
C
C     + + + EXTERNALS + + +
      EXTERNAL  LOGTRN, PRNTXT, LREG6
C
C     + + + END SPECIFICATIONS + + +
C
C     take logs
      CALL LOGTRN ( NOBS, NIDX,  YPR, XIS, FLAG )
C
      IF (FLAG .EQ. 1) THEN
C       Can't procede because of zeros in data.  Add a
C       constant to all values or eliminate the zero points.
        SGRP = 69
        CALL PRNTXT (MESSFL,SCLU,SGRP)
      ELSE IF (FLAG .EQ. 0) THEN
C       compute statistics
        CALL LREG6 (MESSFL, SCLU,
     $              OFILE, NOBS, NIDX, IDXSTA, XIS, YPR,
     $              B,SE,XTXINV,RESID,COOKD,PRED,FLAG)
        IF (FLAG .EQ. 1) THEN
C         too few points to analyze, need more concurrent dates
          SGRP = 13
          CALL PRNTXT (MESSFL, SCLU, SGRP)
        END IF
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   LOGTRN
     I                   ( NPTS, NIDX,
     M                     Y, X,
     O                     ERRFLG )
C
C     + + + PURPOSE + + +
C     The purpose of this routine is to check with user to make a
C     log transformation and do the transformations if the data
C     has no 0.0 or negative numbers.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   NPTS, NIDX, ERRFLG
      REAL      Y(100), X(100,6)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NPTS   - number of concurrent discharges at partial and index
C              stations
C     NIDX   - number of index sites
C     Y      - discharges at partial record station
C     X      - discharges at index sites
C     ERRFLG - error flag
C              0 - no problems
C              1 - found a zero discharge in Y or X
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   N, J
C
C     + + + INTRINSICS + + +
      INTRINSIC ALOG10
C
C     + + + END SPECIFICATIONS + + +
C
      ERRFLG = 0
C
C     check for zero or very nearly zero discharges
      DO 50 N = 1,NPTS
        IF (Y(N) .LT. 1.0E-9) ERRFLG = 1
        DO 40 J = 1,NIDX
          IF (X(N,J) .LT. 1.0E-9) ERRFLG = 1
 40     CONTINUE
 50   CONTINUE
C
      IF (ERRFLG .EQ. 0) THEN
C       ok, take logs
        DO 60 N = 1,NPTS
          Y(N) = ALOG10(Y(N))
          DO 55 J = 1,NIDX
            X(N,J) = ALOG10(X(N,J))
 55       CONTINUE
 60     CONTINUE
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   LREG6
     I                (MESSFL, SCLU,
     I                 OFILE, NOBS, NIDX , IDXSTA, W, Y,
     O                 B, SE, XTXINV, RESID, COOKD, PRED, RETCOD)
C
C     + + + PURPOSE + + +
C     This routine does least squares regression on up to
C     six explanatory variables, print results on a file and returns
C     the predicted values, cook D  and residuals for plotting.
C
C     + + + HISTORY + + +
C     kmf - Apr 2, 2002 - added dummy arguments message file unit and
C                         cluster.  Replaced calls to zbldwr with writes
C                         to prtrow which is passed to q2edit; revised
C                         associated format statements.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   MESSFL, SCLU
      INTEGER   NOBS, NIDX , OFILE, RETCOD, IDXSTA(NIDX)
      REAL    W(100,6), Y(100), PRED(NOBS),RESID(NOBS),COOKD(NOBS)
      DOUBLE PRECISION   B(10,1), SE, XTXINV(10,10)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MESSFL - Fortran unit number of message file
C     SCLU   - cluster number in message file
C     OFILE  - Fortran unit number of output file
C     NOBS   - number of observations
C     NIDX   - number of index stations (max is 6)
C     W      - (nobsxnstat)matrix of data for index stations
C     Y      - response variable
C     B      - regression coefficients
C     SE     - standard error of estimate
C     XTXINV - inverse of XTX matrix
C     RESID  - residual
C     COOKD  - Cook's D statistic
C     PRED   - predicted values
C     RETCOD - 1 if too few points for analysis, else 0
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      NCOLS, I, J, IDF, ID1, L78, L1
      REAL         SB, T, TABS, PVAL
Ckmf  April 1, 2004, added variable
      INTEGER      L3, IRET, SGRP
      CHARACTER*78 PRTROW(100)
      CHARACTER*8  CON
      DOUBLE PRECISION   X(100,10),XB(100,1),Z(100,1),
     &                   XT(10,100),XTZ(10,1),BT(1,10),ZT(1,100),
     &                   T1(1,1),T2(1,1),H(100,100),XTXXT(10,100),
     &                   XTX(10,10),R,VAR,DET,SUM,YBAR,SSREG,SSTOT,
     &                   FRATIO,RADJ,SRESID,HII,SSRES,REGMS
C
C     + + + INTRINSICS + + +
      INTRINSIC   SQRT, SNGL, FLOAT, ABS, DBLE, DSQRT
C
C     + + + FUNCTIONS + + +
      REAL   STUTP
C
C     + + + EXTERNALS + + +
      EXTERNAL   MULTPY, INVERT, STUTP
      EXTERNAL   Q2INIT, Q2SCTF, Q2EDIT
C
C     + + + OUTPUT FORMATS + + +
C     ANOVA table
 1998 FORMAT (     ' ANOVA TABLE',
     $          /, ' SOURCE       DF        SS             MS',
     $             '             F      R SQ   ADJ. R SQ',
     $         //, ( A ) )
 2001 FORMAT (     ' REGRESSION', I5, 2G15.5, 3F10.2 )
 2002 FORMAT (     ' RESIDUAL  ', I5, 2G15.5 )
 2003 FORMAT (     ' TOTAL     ', I5,  G15.5 )
C     regression coefficients table
 2008 FORMAT (///, ' REGRESSION COEFFICIENTS',
     $          /, '   VARIABLE       COEFFICIENT   STANDARD ERROR',
     $             '   T FOR H0:BETA=0    PROB>|T|',
     $         //, ( A ) )
 2020 FORMAT (     2X, I8, 3G18.5, F10.4 )
 2021 FORMAT (     2X, A8, 3G18.5, F10.4 )
C     residual analysis
 2050 FORMAT (     1X, I3, 4G13.5, 2F10.3 )
 2051 FORMAT ( //, ' RESIDUAL ANALYSIS',
     $          /, ' NO.    OBSERVED    PREDICTED     RESIDUAL',
     $             '     STANDARD R   LEVERAGE  COOK D',
     $          /, ( A ) )
C
C     + + + END SPECIFICATIONS + + +
C
      L78 = 78
      L1 = 1
      L3 = 3
C
C     resid=residual
C     sresid=standardized residual
C     hii=leverage
C     t=t statistic
C     pval=p level
C
C     Load matrices
C
      NCOLS=NIDX +1
      SUM=0.0
      DO 10 I=1,NOBS
         X(I,1)=1.0
         X(I,2)=DBLE(W(I,1))
         Z(I,1)=DBLE(Y(I))
         ZT(1,I)=Z(I,1)
         SUM=SUM+Z(I,1)
         IF(NCOLS.GT.2)X(I,3)=DBLE(W(I,2))
         IF(NCOLS.GT.3)X(I,4)=DBLE(W(I,3))
         IF(NCOLS.GT.4)X(I,5)=DBLE(W(I,4))
         IF(NCOLS.GT.5)X(I,6)=DBLE(W(I,5))
         IF(NCOLS.GT.6)X(I,7)=DBLE(W(I,6))
 10   CONTINUE
      YBAR=SUM/NOBS
C
C     Transpose x
C
      DO 20 I=1,NOBS
        DO 15 J=1,NCOLS
          XT(J,I)=X(I,J)
 15     CONTINUE
 20   CONTINUE
C
C     Compute b and XTXinv
C
      CALL MULTPY(XT,X,NCOLS,NOBS,NCOLS,10,10,100,XTX)
      CALL INVERT(NCOLS,10,XTX,0,OFILE,DET,XTXINV)
      CALL MULTPY(XT,Z,NCOLS,NOBS,1,10,10,100,XTZ)
      CALL MULTPY(XTXINV,XTZ,NCOLS,NCOLS,1,10,10,10,B)
C
C     Compute ANOVA table
C
      DO 30 I=1,NCOLS
        BT(1,I)=B(I,1)
 30   CONTINUE
      CALL MULTPY(BT,XTZ,1,NCOLS,1,1,1,10,T1)
      CALL MULTPY(ZT,Z,1,NOBS,1,1,1,100,T2)
      SSREG=T1(1,1)-NOBS*YBAR**2
      SSTOT=T2(1,1)-NOBS*YBAR**2
      SSRES=T2(1,1)-T1(1,1)
      IDF=NOBS-NCOLS
      ID1=NOBS-1
      IF(IDF.GE.1)THEN
        VAR=SSRES/IDF
        SE=DSQRT(VAR)
        R=SSREG/SSTOT
        RADJ=1.0-(FLOAT(NOBS-1)/FLOAT(NOBS-NCOLS))*(1.0-R)
        REGMS=SSREG/NIDX
        FRATIO=REGMS/VAR
C
C       Write ANOVA table to terminal and file
C
Ckmf    ANOVA table, April 1, 2004
C       set up output for ANOVA table to screen and file
C       regression results, residual results, & totals
        WRITE (PRTROW(1),2001) NIDX, SSREG, REGMS, FRATIO, R, RADJ
        WRITE (PRTROW(2),2002) IDF, SSRES, VAR
        WRITE (PRTROW(3),2003) ID1, SSTOT
C
C       ANOVA table to print file
        WRITE (OFILE,1998) (PRTROW(I), I = 1, 3)
C
C       ANOVA table to screen
        SGRP = 110
        CALL Q2INIT ( MESSFL, SCLU, SGRP )
        CALL Q2SCTF ( L1, L78, L3, PRTROW )
        CALL Q2EDIT ( L3, IRET )
Ckmf    end ANOVA table, April 1 revision
C
C       Regression coefficients table
C
Ckmf    Regression coefficients table, April 2, 2004
        SB=SNGL(XTXINV(1,1)*SE**2)
        SB=SQRT(SB)
        T=SNGL(B(1,1))/SB
        TABS=ABS(T)
        PVAL=2.0*STUTP(-TABS,IDF)
        IF(PVAL.LT..0001)PVAL=.0001
        CON='CONSTANT'
C       results
        WRITE (PRTROW(1),2021) CON, B(1,1), SB, T, PVAL
        DO 40 I=2,NCOLS
          SB=SNGL(XTXINV(I,I)*SE**2)
          SB=SQRT(SB)
          T=SNGL(B(I,1))/SB
          TABS=ABS(T)
          PVAL=2.0*STUTP(-TABS,IDF)
          IF(PVAL.LT..0001)PVAL=.0001
          WRITE (PRTROW(I),2020) IDXSTA(I-1), B(I,1), SB, T, PVAL
 40     CONTINUE
C
C       Regression coefficients table to print file
        WRITE (OFILE,2008) (PRTROW(I), I = 1, NCOLS )
C
C       Regression coefficients table to screen
        SGRP = 111
        CALL Q2INIT ( MESSFL, SCLU, SGRP )
        CALL Q2SCTF ( L1, L78, NCOLS, PRTROW )
        CALL Q2EDIT ( NCOLS, IRET )
Ckmf    end Regression coefficients table, April 1 revision
C
C       Residual analysis
C
Ckmf    Residual analysis, April 1, 2004
        CALL MULTPY(XTXINV,XT,NCOLS,NCOLS,NOBS,10,10,10,XTXXT)
        CALL MULTPY(X,XTXXT,NOBS,NCOLS,NOBS,100,100,10,H)
        CALL MULTPY(X,B,NOBS,NCOLS,1,100,100,10,XB)
C
        DO 60 I=1,NOBS
          RESID(I)= SNGL(Z(I,1)- XB(I,1))
          PRED(I)= SNGL(XB(I,1))
          HII=H(I,I)
          SRESID=DBLE(RESID(I))/(SE*DSQRT(1.0-HII))
          COOKD(I)=SNGL(SRESID**2*HII/((1.0-HII)*NCOLS))
          WRITE (PRTROW(I),2050) I, Z(I,1), XB(I,1), RESID(I),
     $                              SRESID, HII, COOKD(I)
 60     CONTINUE
C
C       residual analysis to print file
        WRITE (OFILE,2051) (PRTROW(I), I = 1, NOBS)
C
C       residual analysis to screen
        SGRP = 112
        CALL Q2INIT ( MESSFL, SCLU, SGRP )
        CALL Q2SCTF ( L1, L78, NOBS, PRTROW )
        CALL Q2EDIT ( NOBS, IRET )
Ckmf    end residual analysis, April 1 revision
C
        RETCOD = 0
C
      ELSE
C       not enough points
        RETCOD = 1
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   PSTPLT
     I                   (MESSFL, SCLU, NPTS, YPR, PRED, RESID, COOKD)
C
C     + + + PURPOSE + + +
C     This routine sets up arrays to plot results of the least squares
C     analysis and calls the routine that set ??? and makes the plot.
C
C     + + + HISTSORY + + +
C     kmf - Mar 22, 2004 - added dimensions to declaration
C                          of reals pred, resid, and cookd
C     + + + DUMMY ARGUMENTS + + +
      INTEGER      MESSFL, SCLU, NPTS
Ckmf  March 22, 2004
Ckmf  REAL         PRED, RESID, COOKD, YPR(100)
      REAL         PRED(NPTS), RESID(NPTS), COOKD(NPTS), YPR(100)
Ckmf  end March 22, 2004 modification
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MESSFL - Fortran unit number of message file
C     SCLU   - screen cluster number on message file
C     NPTS   - number of points
C     YPR    - partial record discharges
C     PRED   - predicted values
C     RESID  - residual
C     COOKD  - Cook's D statistic
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      I, J, SGRP, XOPT, YOPT, IRET, CNUM, CBAS
      REAL         Y(100), X(100)
      CHARACTER*1  TITL(9,4)
C
C     + + + EXTERNALS + + +
      EXTERNAL   PLTVRL
      EXTERNAL   Q1INIT, Q1EDIT, QGTCOB, COPYR
C
C     + + + DATA INITIALIZATIONS + + +
      DATA  TITL/'O','b','s','e','r','v','e','d',' ',
     &           'P','r','e','d','i','c','t','e','d',
     &           'R','e','s','i','d','u','a','l',' ',
     &           'C','o','o','k',' ','D',' ',' ',' '/
C
C     + + + END SPECIFICATIONS + + +
C
 10   CONTINUE
C       get variable for y-axis & x-axis
        SGRP = 100
        CALL Q1INIT (MESSFL, SCLU, SGRP)
        CALL Q1EDIT (IRET)
C     if user chose 'Oops', redraw screen
      IF (IRET .EQ. -1) GOTO 10
C
C     retrieve user input
      CNUM = 1
      CBAS = 1
      CALL QGTCOB (CNUM, CBAS, XOPT)
      CBAS = 2
      CALL QGTCOB (CNUM, CBAS, YOPT)
C
      IF (YOPT .EQ. 1) THEN
        CALL COPYR (NPTS,YPR,Y)
      ELSE IF (YOPT .EQ. 2) THEN
        CALL COPYR (NPTS,PRED,Y)
      ELSE IF (YOPT .EQ. 3) THEN
        CALL COPYR (NPTS,RESID,Y)
      ELSE IF (YOPT .EQ. 4) THEN
        CALL COPYR (NPTS,COOKD,Y)
      END IF
C
      IF (XOPT .EQ. 1) THEN
        CALL COPYR (NPTS,YPR,X)
      ELSE IF (XOPT .EQ. 2) THEN
        CALL COPYR (NPTS,PRED,X)
      ELSE IF (XOPT .EQ. 3) THEN
        CALL COPYR (NPTS,RESID,X)
      ELSE IF (XOPT .EQ. 4) THEN
        CALL COPYR (NPTS,COOKD,X)
      END IF
      I = YOPT
      J = XOPT
      CALL PLTVRL  (MESSFL, SCLU, NPTS, Y, X, TITL(1,I), TITL(1,J))
C
      RETURN
      END
C
C
C
      SUBROUTINE   PLTVRL
     I                   (MESSFL, SCLU, MAXP, YPT, XPT, YTITL, XTITL)
C
C     + + + PURPOSE + + +
C     calls routines to make a plot of the results
C     from the Thomas-Stedinger technique.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER      MESSFL, SCLU, MAXP
      REAL         YPT(MAXP), XPT(MAXP)
      CHARACTER*1  XTITL(9), YTITL(9)
C
C     + + + ARGUMENT DEFINITION + + +
C     MAXP   - number of points in YPT and XPT to plot
C     YPT    - array of Y values to plot
C     XPT    - array of X values to plot
C     YTITL  - y label for plot
C     XTITL  - x label for plot
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     I, I1, L9, L40, L80, SGRP, IPOS, RETCOD,
     $            GWHICH(2), GTICS(4), GBVALF(4), GCTYP(2), GCOLOR(2),
     $            ICHK(7), GPATRN(2), GTRANF(2), GXTYP,
     $            GYTYP(2), GLNTYP(2), GSYMBL(2), AXTYP, IVAR, NUMCRV,
     $            NUMVAR, ICRV, YVAR, XVAR, WSID, ICLOS, IWAIT, MORE,
     $            WNDFLG, IND, DEVTYP, DEVCOD, WNDOPN, WNDCHK, CMPTYP
      REAL        SIZEL, XPAGE, YPAGE, XPHYS, YPHYS, XLEN, YLEN, ALEN,
     $            GVMIN(2), GVMAX(2), PLMN(4), PLMX(4)
      CHARACTER*1 OLAB(20,2), BLNK, XLAB(80), YLAB(80), YXLAB(80),
     $            CLAB1(20,2), TITL1(240)
      CHARACTER*8 WNAM(2)
C
C     + + + EXTERNALS + + +
      EXTERNAL    GPINIT, GPNCRV, GPDATR, GPCURV, GPVAR, SCALIT, GPSCLE
      EXTERNAL    GPLABL, GPSIZE, GPWCXY, PSTUPW, PLTONE, PDNPLT, PROPLT
      EXTERNAL    GPDEVC, GGDVTY, GPLBXB
      EXTERNAL    ZIPI, ZIPC, CHRCHR
C
C     + + + DATA INITIALIZATIONS + + +
      DATA  SIZEL, XPAGE, YPAGE, XPHYS, YPHYS, XLEN, YLEN, ALEN, AXTYP
     $     / 0.12,  10.0,  8.0,    1.5,   1.5,  7.5,  5.0,  0.0,     1/
      DATA  L9, L40, L80, BLNK, TITL1
     $     / 9,  40,  80,  ' ', 240*' ' /
      DATA  OLAB /'Y','-','a','x','i','s',14*' ',
     $            'X','-','a','x','i','s',14*' '/
C
C     + + + END SPECIFICATIONS + + +
C
      I1 = 1
      I = 7
      CALL ZIPI (I, I1, ICHK)
C     don't allow modification of axes
      ICHK(2) = 2
      WNAM(1) = 'Modify'
      WNAM(2) = 'PASPM'
C     initialize graphics buffer
      CALL GPINIT
C     set plotting device to screen
      DEVTYP = 1
      IND = 40
      CALL ANPRGT (IND,DEVCOD)
      CALL ANPRGT ( I1, CMPTYP )
      CALL GPDEVC (DEVTYP,DEVCOD)
C
C     set legend to blanks
      CALL ZIPC (L40, BLNK, CLAB1)
C     assign axes labels
      CALL ZIPC (L80, BLNK, XLAB)
      CALL ZIPC (L80, BLNK, YLAB)
      CALL CHRCHR (L9, XTITL, XLAB)
      CALL CHRCHR (L9, YTITL, YLAB)
C
C     number of curves and variables
      NUMCRV = 1
      NUMVAR = 2
      CALL GPNCRV (NUMCRV, NUMVAR)
C
C     save the Y data
      IVAR = 1
      IPOS= 1
      CALL GPDATR (IVAR, IPOS, MAXP, YPT, RETCOD)
C     save the X data
      IVAR = 2
      IPOS= IPOS+ MAXP
      CALL GPDATR (IVAR, IPOS, MAXP, XPT, RETCOD)
C
C     save curve info for xy plot
      GCTYP(1) = 6
C     set line type to none
      GLNTYP(1) = 0
C     set symbol type to '+'
      GSYMBL(1) = 2
C     black/white color
      GCOLOR(1)= 1
C     no pattern
      GPATRN(1)= 0
      CALL GPCURV (GCTYP, GLNTYP, GSYMBL, GCOLOR, GPATRN, CLAB1)
C
C     save data ranges
      GWHICH(1)= 1
      GWHICH(2)= 4
      GVMIN(1) = 1.0E30
      GVMAX(1) = -1.0E30
      GVMIN(2) = 1.0E30
      GVMAX(2) = -1.0E30
      DO 10 I= 1, MAXP
        IF (YPT(I).GT.GVMAX(1)) GVMAX(1)= YPT(I)
        IF (YPT(I).LT.GVMIN(1)) GVMIN(1)= YPT(I)
        IF (XPT(I).GT.GVMAX(2)) GVMAX(2)= XPT(I)
        IF (XPT(I).LT.GVMIN(2)) GVMIN(2)= XPT(I)
 10   CONTINUE
C     no transformation (data was already log-transformed)
      GTRANF(1) = 1
      GTRANF(2) = 1
      CALL GPVAR (GVMIN, GVMAX, GWHICH, GTRANF, OLAB)
C
C     assign variables to first (only) curve
      ICRV = 1
      YVAR = 1
      XVAR = 2
      CALL GPWCXY (ICRV, YVAR, XVAR)
C
C     scale data, Y then X
      CALL SCALIT (AXTYP, GVMIN(1), GVMAX(1),
     O             PLMN(1), PLMX(1))
      CALL SCALIT (AXTYP, GVMIN(2), GVMAX(2),
     O             PLMN(4), PLMX(4))
C
C     set up axis scaling
      GTICS(1) = 10
      GTICS(4) = 10
      CALL ZIPI (4, 1, GBVALF)
      CALL GPSCLE (PLMN, PLMX, GTICS, GBVALF)
C     define axes
      GXTYP = AXTYP
      GYTYP(1) = AXTYP
      GYTYP(2) = 0
      CALL GPLABL (GXTYP, GYTYP, ALEN, YLAB, YXLAB, CLAB1, TITL1)
C     assign x-axis label
      CALL GPLBXB (XLAB)
C     set up sizes
      CALL GPSIZE (SIZEL, XPAGE, YPAGE, XPHYS, YPHYS, XLEN, YLEN, ALEN)
C
C     make the plot
      WNDCHK = 0
      WNDOPN = 1
      WSID = 1
      IWAIT = 0
      IF (CMPTYP .EQ. 1) THEN
        ICLOS = 1
      ELSE
        ICLOS = 0
      END IF
      CALL PSTUPW (WSID, RETCOD)
      CALL PLTONE
      CALL PDNPLT (WSID, ICLOS, IWAIT)
 200  CONTINUE
C       allow user to modify plot:  replot, modify, done
        SGRP = 101
        CALL QRESP (MESSFL, SCLU, SGRP, MORE)
        IF (MORE .EQ. 1) THEN
C         replot
          WSID = 1
          IWAIT = 0
          IF (WNDCHK .EQ. 1) THEN
C           device type or code changed close and repopen
            ICLOS = 1
            CALL PDNPLT ( WSID, ICLOS, IWAIT )
            WNDCHK = 0
            WNDOPN = 1
          END IF
          CALL PSTUPW ( WSID, RETCOD )
          WNDOPN = 1
          CALL PLTONE
          IF (CMPTYP .EQ. 1) THEN
C           pc, always clos workstation after plot
            ICLOS = 1
            WNDOPN = 0
          ELSE
C           for other types, just deactivate the workstation
            ICLOS = 0
          END IF
          CALL PDNPLT ( WSID, ICLOS, IWAIT )
        ELSE IF (MORE .EQ. 2) THEN
C         modify specs
          CALL PROPLT (MESSFL, ICHK, WNAM, WNDFLG )
          IF (WNDCHK .EQ. 0  .AND.  WNDOPN .EQ. 1) WNDCHK = WNDFLG
        END IF
      IF (MORE .LE. 2) GO TO 200
C
      IF (CMPTYP .NE. 1) THEN
C       not pc, need to close workstation
        WSID = 1
        ICLOS = 1
        IWAIT = 0
        CALL PDNPLT ( WSID, ICLOS, IWAIT )
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   PSTCOM
     I                  (MESSFL, SCLU, WDMSFL, OFILE, NSTAT, NIDX,
     I                   DSNI, DSNP, B, SE, XTXINV)
C
C     + + + PURPOSE + + +
C     This routine does Stedinger-Thomas calculations and puts
C     attributes on partial record data sets.
C
C     + + + HISTORY + + +
C     kmf Mar 22, 2004 - added dimensions to double precision
C                        declarations of b and xtxinv
C     kmf Oct 22, 2004 - deleted informational print to screen
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   MESSFL, SCLU, WDMSFL, NSTAT, NIDX, OFILE
      INTEGER   DSNP(NSTAT), DSNI(1,NSTAT)
Ckmf  March 22, 2004
Ckmf  DOUBLE PRECISION   B, SE, XTXINV
      DOUBLE PRECISION   B(10,1), SE, XTXINV(10,10)
Ckmf  end of March 22, 2004 revision
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MESSLF - Fortran unit number of users message file
C     SCLU   - cluster number on message file
C     WDMSFL - Fortran unit number of WDM file
C     OFILE  - Fortran unit number of output file
C     NSTAT  - number of low flow statistics
C     NIDX   - number of index stations
C     DSNI   - data set numbers at index stations
C     DSNP   - data set numbers for partial record stations
C     B      - regression coefficients
C     SE     - standard error of estimate
C     XTXINV - inverse of XTX matrix
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   N, RETCOD, NY
      REAL      Z(100,6)
C
C     + + + EXTERNALS + + +
      EXTERNAL   GETZZS, THOMAS
C
C     + + + END SPECIFICATIONS + + +
C
C     begin loop to compute and store attributes for each
C     low flow statistic
      DO 70 N = 1,NSTAT
C       get annual series for index sites
C       take logs, drop zeros, only for concurrent
        CALL GETZZS (WDMSFL, DSNI(1,N), NIDX, NY, Z)
C
        CALL THOMAS (OFILE, MESSFL, SCLU, WDMSFL, DSNP(N),
     &               NY, NIDX, Z, B, SE, XTXINV, RETCOD)
 70   CONTINUE
C
      RETURN
      END
C
C
C
      SUBROUTINE   GETZZS
     I                   (WDMSFL, DSNI, NSTAT,
     O                    NY, Z)
C
C     + + + PURPOSE + + +
C     This routine gets annual series for index sites for a common
C     period of non-zero flows and transforms data to logs base 10.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   WDMSFL, DSNI(6), NSTAT,NY
      REAL      Z(100,6)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     WDMSFL  - Fortran unit number of users message file
C     DSNI   - data set numbers of index sites
C     NSTAT  - number of index sites
C     NY     - number of non-zero concurrent years
C     Z      - log of flows for concurrent period
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I, CNT, N, FLG, GPFLG, TDSFRC, QFLG, DTRAN, L1, L2,
     &          EDAT(6,6), SDAT(6,6), EDATE(6), SDATE(6), TS, TU,
     &          RETCOD
C
C     + + + INTRINSICS + + +
      INTRINSIC   ALOG10
C
C     + + + EXTERNALS + + +
      EXTERNAL   WDTGET, TIMDIF, DLIMIT, WTFNDT
C
C     + + + DATA INITIALIZATIONS + + +
      DATA  L1, L2, TS, TU, DTRAN, QFLG, GPFLG, TDSFRC
     $     / 1,  2,  1,  6,     0,   30,     0,      0 /
C
C     + + + END SPECIFICATIONS + + +
C
      DO 20 I = 1,NSTAT
C       get start and end dates for each data set
        CALL WTFNDT(WDMSFL,DSNI(I),GPFLG,TDSFRC,SDAT(1,I),EDAT(1,I),
     &              RETCOD)
 20   CONTINUE
C
C     find common period
      CALL DLIMIT (SDAT,NSTAT,L2,SDATE)
      CALL DLIMIT (EDAT,NSTAT,L1,EDATE)
      CALL TIMDIF (SDATE,EDATE,TU,TS,NY)
C
      IF (NY .GT. 0) THEN
C
C       get the data
        DO 30 I = 1,NSTAT
          CALL WDTGET (WDMSFL,DSNI(I),TS,SDATE,NY,DTRAN,QFLG,TU,
     &                 Z(1,I),RETCOD)
 30     CONTINUE
C
C       screen for zeros and take logs
        CNT = 0
        DO 50 N = 1,NY
          FLG = 0
          DO 45 I = 1,NSTAT
            IF (Z(N,I) .LE. 0.0) FLG = 1
 45       CONTINUE
          IF (FLG .EQ. 0) THEN
C           ok year
            CNT = CNT + 1
            DO 40 I = 1,NSTAT
              Z(CNT,I) = ALOG10(Z(N,I))
 40         CONTINUE
          END IF
 50     CONTINUE
C
        NY = CNT
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   THOMAS
     I                  (OFILE,MESSFL,SCLU,WDMSFL,DSN,NY,NIDX,
     I                   Z,B,SE,XTXINV,
     O                   RETCOD)
C
C     + + + PURPOSE + + +
C     This subroutine uses the Stedinger-Thomas method to estimate
C     low flow characterictics from base-flow measurements correlated
C     with concurrent data at up to six index gaging stations.  The
C     mean, std dev, skew, equivalent years, and flow statistics
C     are put on the WDM data set.
C
C     + + + HISTORY + + +
C     kmf - Apr 4, 2002 - Replaced calls to zbldwr with writes
C                         to prtrow which is passed to q2edit;
C                         revised associated format statements.
C                         Replaced multiple calls to write
C                         attributes to wdm with singe call to
C                         write them in a loop.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   NY, NIDX, OFILE, DSN, RETCOD, WDMSFL, MESSFL, SCLU
      REAL      Z(100,6)
      DOUBLE PRECISION   B(10,1), SE, XTXINV(10,10)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     OFILE  - Fortran unit number of output file
C     MESSFL - Fortran unit number of GLSNET message file
C     SCLU   - cluster number for message file
C     WDMSFL - Fortran unit number of users WDM file
C     DSN    - data set number of partial record site
C     NY     - number of concurrent years for index sites
C     NIDX   - number of index stations
C     Z      - (ny x nidx) matrix of annual series for index stations
C     B      - vector of regression coefficient from LREG6
C     SE     - standard error from LREG6
C     XTXINV - inverse of XTX matrix from LREG6
C     RETCOD - return code, 0 if all ok
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I, J, IH, IG, L1, L8, L10, L48, L78, NCOLS, SGRP,
     $          SATYP, SALEN, SAIND, LENATN(8), INDEX(8), ISTANO,
     $          RETC(10), ERRFLG, RETFLG
      REAL      SKEW(10), R(4), AK(4), AVGSKU, SUM4, P, ATTVAL(8)
      DOUBLE PRECISION   BT(1,10),X(100,10),COV,VXX(10,10),VXXT(10,10),
     &                   XMU(10,1),XMUT(1,10),YMU(1,1),WORK(10,1),
     &                   T1(1,1),VS(10,10),VSB(10,1),A(10,10),
     &                   XT(10,100),YHAT(4),VYHAT(4),VMU,SUM,SUM2,SUM3,
     &                   SD,S1,TRACE,TERM1,TERM2,YVAR,YSD,HARD,
     &                   EQYRS(4),VARV,        DNY
      CHARACTER*4    CTYPE
      CHARACTER*6    ATNAM(8)
      CHARACTER*78   PRTROW(10)

C     + + + INTRINSICS + + +
      INTRINSIC   SNGL, DSQRT, DBLE
C
C     + + + FUNCTIONS + + +
      REAL   WILFRT, GAUSAB
C
C     + + + EXTERNALS + + +
      EXTERNAL   WILFRT, MULTPY, GAUSAB
      EXTERNAL   WDBSGX, WDBSGC, WDBSGI, WDBSAR
      EXTERNAL   Q2INIT, Q2SCTF, Q2EDIT
      EXTERNAL   Q1INIT, QSETI,  QSETR, QSETCT, Q1EDIT
C
C     + + + DATA INITIALIZATIONS + + +
      DATA  L1, L8, L10, L48, L78, LENATN, R
     $     / 1,  8,  10,  48,  78,    8*6, 2.0,5.0,10.0,20.0 /
      DATA  ATNAM  / 'L  002', 'L  005', 'L  010', 'L  020',
     $               'MEANND', 'SDND  ', 'SKEW  ', 'YRSLOW' /
      DATA  INDEX  /     0,        0,        0,        0,
     $                 280,      281,      282,      219    /
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT (1X, G15.5, G10.2, 3G15.5 )
 2001 FORMAT (    ' ESTIMATED FLOW CHARACTERISTICS AT PARTIAL-RECORD',
     $            ' SITE', I12 )
 2002 FORMAT (    ' MEAN=', G13.5 )
 2003 FORMAT (    ' STD DEV=', G13.5 )
 2004 FORMAT (    ' SKEW COEF.=', G13.5 )
 2005 FORMAT (    ' RETURN PERIOD      Y          LOG Y          ',
     $            'VARIANCE     EQUIVALENT YEARS' )
 2020 FORMAT ( //, A,
     $         //, A, /, A, /, A,
     $         //,(A) )
C
C     + + + END SPECIFICATIONS + + +
C
C     load data
C
      NCOLS=NIDX +1
      DO 11 I=1,NY
        X(I,1)=1.0
        X(I,2)=DBLE(Z(I,1))
        IF(NCOLS.GT.2)X(I,3)=DBLE(Z(I,2))
        IF(NCOLS.GT.3)X(I,4)=DBLE(Z(I,3))
        IF(NCOLS.GT.4)X(I,5)=DBLE(Z(I,4))
        IF(NCOLS.GT.5)X(I,6)=DBLE(Z(I,5))
        IF(NCOLS.GT.6)X(I,7)=DBLE(Z(I,6))
 11   CONTINUE
C
C     Center data
C
      DNY = DBLE(NY)
      DO 30 J=1,NCOLS
         SUM=0.0
         SUM2=0.0
         SUM3=0.0
         DO 20 I=1,NY
            SUM=SUM+X(I,J)
            SUM2=SUM2+X(I,J)**2
            SUM3=SUM3+X(I,J)**3
 20      CONTINUE
         XMUT(1,J)=SUM/DNY
         XMU(J,1)=SUM/DNY
         IF(J.GT.1)THEN
            SD=DSQRT((SUM2-SUM**2/DNY)/(DNY-1.0))
            S1=SUM3*DNY**2-3.0*DNY*SUM*SUM2+2.0*SUM**3
            SKEW(J-1)=SNGL(S1/(SD**3*DNY*(DNY-1.0)*(DNY-2.0)))
         END IF
         DO 22 I=1,NY
            X(I,J)=X(I,J)-SUM/DNY
 22      CONTINUE
 30   CONTINUE
C
C     Transpose x
C
      DO 40 I=1,NY
        DO 35 J=1,NCOLS
          XT(J,I)=X(I,J)
 35     CONTINUE
 40   CONTINUE
C
C     Compute variance matrices
C
      CALL MULTPY(XT,X,NCOLS,NY,NCOLS,10,10,100,VXX)
      DO 50 I=1,NCOLS
        BT(1,I)=B(I,1)
        DO 45 J=1,NCOLS
          VXX(I,J)=VXX(I,J)/(DNY-1.0)
 45     CONTINUE
 50   CONTINUE
C
C     Compute ymu yvar
C
      CALL MULTPY(XMUT,B,1,NCOLS,1,1,1,10,YMU)
      CALL MULTPY(VXX,B,NCOLS,NCOLS,1,10,10,10,WORK)
      CALL MULTPY(BT,WORK,1,NCOLS,1,1,1,10,T1)
      CALL MULTPY(VXX,XTXINV,NCOLS,NCOLS,NCOLS,10,10,10,VS)
      TRACE=0.0
C     Following comment out because TRACE is greater than 1.0 in
C     some cases with few partial record measurements and small
C     variance for those mesaurements.  Also it is not supposed
C     to be significant for large data sets.
C     DO 55 I=1,NCOLS
C       TRACE=TRACE+VS(I,I)
C55   CONTINUE
      TERM2=SE**2*(1.0-TRACE)
      YVAR=TERM2+T1(1,1)
C
C     Compute k values
C
      SUM4=0.0
      DO 60 I=1,NIDX
        SUM4=SUM4+SKEW(I)
 60   CONTINUE
      AVGSKU=SUM4/NIDX
      R(1)=2.0
      R(2)=5.0
      R(3)=10.0
      R(4)=20.0
C
C     Compute var
C
      CALL MULTPY(XTXINV,XMU,NCOLS,NCOLS,1,10,10,10,WORK)
      CALL MULTPY(XMUT,WORK,1,NCOLS,1,1,1,10,T1)
      TERM1=SE**2*T1(1,1)
      CALL MULTPY(VXX,B,NCOLS,NCOLS,1,10,10,10,WORK)
      CALL MULTPY(BT,WORK,1,NCOLS,1,1,1,10,T1)
      TERM2=T1(1,1)/DNY
      VMU=TERM1+TERM2
C
C     Compute COV
C
      CALL MULTPY(XTXINV,VXX,NCOLS,NCOLS,NCOLS,10,10,10,VS)
      CALL MULTPY(VS,B,NCOLS,NCOLS,1,10,10,10,VSB)
      CALL MULTPY(XMUT,VSB,1,NCOLS,1,1,1,10,T1)
      COV=2.0*SE**2*T1(1,1)
C
C     Compute VAR(var)
C
      DO 75 I=1,NCOLS
        DO 74 J=1,NCOLS
          DO 73 IH=1,NCOLS
            DO 72 IG=1,NCOLS
              A(I,J)=B(IH,1)*B(IG,1)*(VXX(I,IH)*VXX(IG,J)+
     1               VXX(I,J)*VXX(IH,IG))
 72         CONTINUE
 73       CONTINUE
          VXXT(I,J)=VXX(J,I)
 74     CONTINUE
 75   CONTINUE
      CALL MULTPY(VXXT,VSB,NCOLS,NCOLS,1,10,10,10,WORK)
      CALL MULTPY(BT,WORK,1,NCOLS,1,1,1,10,T1)
      TERM1=SE**4*(2.0/(NY-NCOLS)+4.0*T1(1,1))
      CALL MULTPY(A,B,NCOLS,NCOLS,1,10,10,10,WORK)
      CALL MULTPY(BT,WORK,1,NCOLS,1,1,1,10,T1)
      TERM2=T1(1,1)/(DNY-1.0)
      VARV=TERM1+TERM2
C
C     get station id of partial record site (ISTAID)
      SAIND = 51
      SALEN = 1
      CALL WDBSGI (WDMSFL,DSN,SAIND,SALEN,ISTANO,RETFLG)
      IF (RETFLG .NE. 0) ISTANO = 0
C
C     get TSTYPE
      SAIND = 1
      SALEN = 4
      CALL WDBSGC (WDMSFL,DSN,SAIND,SALEN,CTYPE,
     &             RETCOD)
C
C     identify return period attributes
      DO 80 I = 1, 4
C       get index numbers
        ATNAM(I)(2:3) = CTYPE(3:4)
        CALL WDBSGX (MESSFL,
     M               ATNAM(I),
     O               INDEX(I), SATYP, SALEN )
   80 CONTINUE
C
      YSD = DSQRT(YVAR)
      DO 85 I=1,4
C       loop for each return period (2- 5- 10- 20-)
        P=1.0/R(I)
        AK(I)=WILFRT(AVGSKU,GAUSAB(P),ERRFLG)
        YHAT(I)=YMU(1,1)+DBLE(AK(I))*DSQRT(YVAR)
        ATTVAL(I) = 10.0**(SNGL(YHAT(I)))
        VYHAT(I)=VMU+DBLE(AK(I))**2/(4.0*YVAR)*VARV+DBLE(AK(I))/
     &           DSQRT(YVAR)*COV
        HARD=1.0+.5*DBLE(AK(I)**2)
        EQYRS(I)=HARD*YVAR/VYHAT(I)
        IF (EQYRS(I) .GT. DBLE(NY)) EQYRS(I) = DBLE(NY)
        IF (EQYRS(I) .LT. DBLE(3))  EQYRS(I) = DBLE(3)
        WRITE (PRTROW(I+6),2000) R(I), ATTVAL(I), YHAT(I),
     $                           VYHAT(I), EQYRS(I)
 85   CONTINUE
C
C     Mean, Std Dev, Skew, Years of low flow
      ATTVAL(5) = SNGL(YMU(1,1))
      ATTVAL(6) = SNGL(YSD)
      ATTVAL(7) = AVGSKU
      ATTVAL(8) = SNGL(EQYRS(1))
C
C     Output results
C
C     set up for print and screen
      WRITE (PRTROW(1),2001) ISTANO
      WRITE (PRTROW(2),2002) YMU(1,1)
      WRITE (PRTROW(3),2003) YSD
      WRITE (PRTROW(4),2004) AVGSKU
      PRTROW(5) = ' '
      WRITE (PRTROW(6),2005)
C
C     estimated flow characteristics table to print file
      WRITE (OFILE,2020) (PRTROW(I), I = 1, 10)
C
C     estimated flow characteristics table to screen
      SGRP = 120
      CALL Q2INIT ( MESSFL, SCLU, SGRP )
      CALL Q2SCTF ( L1, L78, L10, PRTROW )
      CALL Q2EDIT ( L10, RETCOD )
C
C     estimated flow characteristics to wdm file
      DO 100 I = 1, L8
C       save attributes to data set
        CALL WDBSAR ( WDMSFL, DSN, MESSFL, INDEX(I), L1, ATTVAL(I),
     O                RETC(I) )
 100  CONTINUE
C
C     summary of attribute processing to screen
      SGRP = 121
      CALL Q1INIT ( MESSFL, SCLU, SGRP )
      RETC(9)  = ISTANO
      RETC(10) = DSN
      CALL QSETI  ( L10, RETC )
      CALL QSETR  ( L8, ATTVAL )
      CALL QSETCT ( L8, LENATN, L48, ATNAM )
      CALL Q1EDIT ( RETCOD )
C
      RETURN
      END
C
C
C
      SUBROUTINE   PSTYRS
     I                   (MESSFL, SCLU, WDMSFL, NSTAT, DSNP, NPTS, IYR)
C
C     + + + PURPOSE + + +
C     Puts an annual time series on a set of data sets, putting a value\
C     of 1.0 for each year specified in the IYR array.
C
C     + + + HISTORY + + +
C     kmf - May 19, 2004 - Replaced calls to pmxtxi with writes to
C                          prtrow which is passed to q2edit.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER      MESSFL, SCLU, WDMSFL, NSTAT, DSNP(NSTAT), NPTS,
     $             IYR(100)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MESSFL - Fortran unit number of message file
C     SCLU   - screen cluster number on message file
C     WDMSFL - Fortran unit number of wdm file
C     NSTAT  - number of annual series for partial record station
C              in buffer
C     DSNP   - array containing up to 12 data sets of annual low-flow
C              series to add partial record estimates
C     NPTS   - number of concurrent discharges
C     IYR    - calendar year of discharge measurement
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      I, K, L100, N, NY, TS, TU, DTOVWR, SDATE(6), QFLAG,
     $             SGRP, RETCOD, IRET, L1, L78, NPRT, KOUNT
      REAL         RNUM(100), RDEF
      CHARACTER*78  PRTROW(12)
C
C     + + + EXTERNALS + + +
      EXTERNAL     WDTPUT
      EXTERNAL     Q2INIT, Q2SCTF, Q2EDIT
      EXTERNAL     ASRTIP,ZIPR
C
C     + + + OUTPUT FORMATS + + +
 2001 FORMAT ( I6, '  Time series was NOT added, retcod =', I5,
     $             '           !! WARNING !!' )
 2002 FORMAT ( I6, '  Beginning in', I5, ', 1.0 was added for',
     $             I4, ' of', I4, ' years.' )
C     -----------------------------------------------------------------
C     Beginning in xxxx, 1.0 was added for xxx of xxx years.
C     Time series was NOT added, retcod = xxxx
C     Indicator value (1.0) added for xxx years beginning in xxxx
C     xxx years added beginning in xxxx, 
C     Indicator (1.0) added for xxx of xxxx years beginning in xxxx
C     Indicator (1.0) added for xxx years from xxxx to xxxx
C
C     + + + DATA INITIALIZATIONS + + +
      DATA TS, TU, DTOVWR, QFLAG, L1, L78, L100, SDATE,       RDEF
     $    / 1,  6,      1,     0,  1,  78,  100, 0,1,1,0,0,0, -999999./
C
C     + + + END SPECIFICATIONS + + +
C
C     sort years
      CALL ASRTIP (NPTS,IYR)
      CALL ZIPR (L100,RDEF,RNUM)
      NY = IYR(NPTS) - IYR(1) + 1
      SDATE(1) = IYR(1)
      KOUNT = 0
      DO 10 I = 1, NPTS
        K = IYR(I) - IYR(1) + 1
        IF (RNUM(K) .LT. 0.0) KOUNT = KOUNT + 1
        RNUM(K) = 1.0
 10   CONTINUE
C
      NPRT = 0
      DO 20 N = 1, NSTAT
        CALL WDTPUT (WDMSFL, DSNP(N), TS, SDATE, NY, DTOVWR,
     I               QFLAG, TU, RNUM,
     O               RETCOD)
        IF (RETCOD .NE. 0) THEN
C         Time series was not added for data set &. Error code &.
          NPRT = NPRT + 1
          WRITE (PRTROW(NPRT),2001) DSNP(N), RETCOD
        ELSE
C         1.0 added for & years for time-series data set &
          NPRT = NPRT + 1
          WRITE (PRTROW(NPRT),2002) DSNP(N), SDATE(1), KOUNT, NY
        END IF
 20   CONTINUE
C
C     summary of put results to screen
      SGRP = 91
      CALL Q2INIT ( MESSFL, SCLU, SGRP )
      CALL Q2SCTF ( L1, L78, NPRT, PRTROW )
      CALL Q2EDIT ( NPRT, IRET )
C
      RETURN
      END
