#!/bin/sh
#
#    check.sh:  Compare (using diff) 'original' versions of files to
#               files in current directory.  Used most often to compare
#               expected files to actual test output.
#   arguments:  DirO - directory containing original, expected versions
#                      defaults to ../data
#               Name - prefix of file names
#                      defaults to test
#  file names:  naming convention is "nameN.sfx", where:
#               name - "test" by default, or supplied as second argument
#                  N - test numbers, see Test below
#                sfx - suffix for file names, see Sufx below
#
#     history:  3/14/94  kmflynn  modified version of check.sh

  DirO=${1-../data}             # by default, data in parallel directory
  Name=${2-test}                # by default, file prefix is test

# output file for differences
  checkout='check.log'
# define range of tests and sufixes to be considered
  Tests='1 1a 2 3 4'
  Sufix='out ot1 ot2 ot3 exp err era erc erd ps'
  DfOpt='-b'

# delete old file and write heading to file
  if [ -f $checkout ] ; then rm $checkout ; fi
  echo                                                        | tee $checkout
  echo                                                        | tee -a $checkout
  echo                                                        | tee -a $checkout
  echo "Comparing original output files and new output files" | tee -a $checkout
  echo "Original files in: " $DirO                            | tee -a $checkout
  echo "             date: " `date`                           | tee -a $checkout
  echo "  "                                                   | tee -a $checkout
  echo " Note:  In checking the compare differences, the"     | tee -a $checkout
  echo "        values for attribute DATMOD will differ in"   | tee -a $checkout
  echo "        some export files (.exp) and the values of"   | tee -a $checkout
  echo "        attribute DATCRE and DATMOD may differ in"    | tee -a $checkout
  echo "        all export files.  The value for creation"    | tee -a $checkout
  echo "        date will differ in postscript (.ps) files."  | tee -a $checkout
  echo                                                        | tee -a $checkout

  for Test in $Tests ; do
     for Sufx in $Sufix ; do
        if [ -f $DirO/$Name$Test.$Sufx -a -f $Name$Test.$Sufx ] ; then
           # both original and new output files exist, compare
           echo "_________________________________________________________ " \
                $Name$Test.$Sufx | tee -a $checkout
           if diff $DfOpt $DirO/$Name$Test.$Sufx $Name$Test.$Sufx >> $checkout
              then
              echo FILES ARE IDENTICAL | tee -a $checkout
           else
              echo FILES DIFFER:  see file $checkout for differences
           fi
        fi
     done
  done

  if [ -f $DirO/test.out -a -f test.out ] ; then
     # both original and new output files exist, compare
     echo "_________________________________________________________ " \
          test.out | tee -a $checkout
     if diff $DfOpt $DirO/test.out test.out >> $checkout
        then
        echo FILES ARE IDENTICAL | tee -a $checkout
     else
        echo FILES DIFFER:  see file $checkout for differences
     fi
  fi

# end of shell
