#!/bin/sh
# test.sh -- run glsnet test data sets
#
# Usage: test.sh [start [stop [size] ] ]
#
#        where: start = starting test number
#                stop = ending test number (may be same as start)
#                size = max no of stations glsnet dimensioned for
#                       valid values are 300 (default), 600, and 700
#
# History: 95/12/08 kmflynn
#          97/06/11 kmflynn  add selection of different dimensions
#          00/11/08 kmflynn

# pathnames
  WrdA=/usr/opt/wrdapp
  GlsN=$WrdA/glsnet4.0
  Anne=$WrdA/annie4.1

#*******************************************************************
#*****   Some things may need to be changed below this line.   *****
#*******************************************************************

# glsnet may be provided in three different sizes,
# dimensioned for up to 300, 600, or 700 stations.
# default size is 300
  Size=${3-300}

#*******************************************************************
#***** You should not need to modify anything below this line. *****
#*******************************************************************

  Data=$GlsN/data
  Chck=$GlsN/test

#
  exec 2>&1                          # stderr shows up in .out file
  Start=${1-1}                       # by default, start at 1
  Stop=${2-4}                        # by default, stop at 4
                                     # 1 - texas peaks
                                     # 2 - virginia low flows
                                     # 3 - virginia 2 low flows
                                     # 4 - texas 2 peaks
#
# begin test runs
  if [ -f test.out ] ; then rm test.out ; fi
  echo                                                        | tee    test.out
  echo ______________________________________________________ | tee -a test.out
  echo                                                        | tee -a test.out
  echo " `date` "                                             | tee -a test.out
  echo                                                        | tee -a test.out
  echo " Processing glsnet test runs $Start to $Stop"         | tee -a test.out
  echo                                                        | tee -a test.out
  echo " glsnet program from: $GlsN"                          | tee -a test.out
  echo "      dimensioned to: $Size"                          | tee -a test.out
  echo "  annie program from: $Anne"                          | tee -a test.out
  echo "      test data from: $Data"                          | tee -a test.out
  echo

  Test=$Start
  while [ $Test -ge $Start -a $Test -le $Stop -a $Test -le 4 ] ; do
#    where should the data come from
     if [ $Test -eq 1 -o $Test -eq 4 ] ; then
#      Texas peaks
       Name=tx
       if [ $Test -eq 1 ] ; then
         NameWdm=tx
       else
         NameWdm=tx2
       fi
     else
#      Virginia low flows
       Name=va
       ln -s $Data/test$Test.pr1 test$Test.pr1
       ln -s $Data/test$Test.pr2 test$Test.pr2
       Name=va
       if  [ $Test -eq 2 ] ; then
         NameWdm=va
       else
         NameWdm=va2
       fi
     fi

#    each requested test run
     echo
     echo
     echo ___________________________________________________ | tee -a test.out
     echo "Test run number $Test - $Name"                     | tee -a test.out
#    echo "                $Name"                             | tee -a test.out
#    remove old files
     for File in out ot1 ot2 exp era err erc erd ; do
       if [ -f test$Test.$File ] ; then rm test$Test.$File ; fi
     done

#    get or build a wdm file
     if [ -f $NameWdm.wdm ] ; then rm $NameWdm.wdm ; fi
     if [ -f $Name.exp ]    ; then rm $Name.exp    ; fi
     if [ -f $Data/$Name.wdm ] ; then
        echo "                Copying wdm from $Data"         | tee -a test.out
        echo "                        source = Data/$Name.wdm"| tee -a test.out
        echo "                        target = $NameWdm.wdm"  | tee -a test.out
        cp -p $Data/$Name.wdm $NameWdm.wdm
     else
        echo "                Building the wdm file"          | tee -a test.out
        ln -s $Data/$Name.exp $Name.exp
        $Anne/bin/annie <<-EOT
	@$Data/test$Test.lga
	EOT
        mv ERROR.FIL test$Test.era
        rm $Name.exp
     fi

     if [ -f $Data/test$Test.lgb ] ; then
#      process partial record data
       echo "                Running glsnet (partial record)" | tee -a test.out
       $GlsN/bin/gls$Size.exe <<-EOT
	@$Data/test$Test.lgb
	EOT
       mv ERROR.FIL test$Test.erb
       rm test$Test.pr1
       rm test$Test.pr2
     fi

     echo "                Running glsnet"                    | tee -a test.out
     $GlsN/bin/gls$Size.exe <<-EOT
	@$Data/test$Test.log
	EOT
     mv ERROR.FIL test$Test.err

     if [ -f $Anne/bin/annie -a -f $Data/test$Test.lgd ] ; then
#       export data sets using annie for verification
        echo "                Exporting data from $NameWdm.wdm"  | tee -a test.out
        $Anne/bin/annie <<-EOT
	@$Data/test$Test.lgd
	EOT
        mv ERROR.FIL test$Test.erd
     fi

     echo "                Completed test $Test"              | tee -a test.out

     Test=`expr $Test + 1`
  done

# remove unneeded files
  Files='ANNIE.LOG GLSNET.LOG XPAD.DAT'
  for File in $Files ; do
    if [ -f $File ] ; then rm $File ; fi
  done

  echo                                                        | tee -a test.out
  echo                                                        | tee -a test.out
  echo ______________________________________________________ | tee -a test.out
  echo "Completed glsnet test runs $Start to $Stop"           | tee -a test.out
  echo                                                        | tee -a test.out

# check output against original output in data directory
  $Chck/check.sh $Data

# end of shell
